"""
Python module generated from Java source file com.google.common.math.LongMath

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.math import *
from com.google.common.primitives import UnsignedLongs
from java.math import BigInteger
from java.math import RoundingMode
from typing import Any, Callable, Iterable, Tuple


class LongMath:
    """
    A class for arithmetic on values of type `long`. Where possible, methods are defined and
    named analogously to their `BigInteger` counterparts.
    
    The implementations of many methods in this class are based on material from Henry S. Warren,
    Jr.'s *Hacker's Delight*, (Addison Wesley, 2002).
    
    Similar functionality for `int` and for BigInteger can be found in IntMath and BigIntegerMath respectively. For other common operations on `long`
    values, see com.google.common.primitives.Longs.

    Author(s)
    - Louis Wasserman

    Since
    - 11.0
    """

    @staticmethod
    def ceilingPowerOfTwo(x: int) -> int:
        """
        Returns the smallest power of two greater than or equal to `x`. This is equivalent to
        `checkedPow(2, log2(x, CEILING))`.

        Raises
        - IllegalArgumentException: if `x <= 0`
        - ArithmeticException: of the next-higher power of two is not representable as a `long`, i.e. when `x > 2^62`

        Since
        - 20.0
        """
        ...


    @staticmethod
    def floorPowerOfTwo(x: int) -> int:
        """
        Returns the largest power of two less than or equal to `x`. This is equivalent to `checkedPow(2, log2(x, FLOOR))`.

        Raises
        - IllegalArgumentException: if `x <= 0`

        Since
        - 20.0
        """
        ...


    @staticmethod
    def isPowerOfTwo(x: int) -> bool:
        ...


    @staticmethod
    def log2(x: int, mode: "RoundingMode") -> int:
        """
        Returns the base-2 logarithm of `x`, rounded according to the specified rounding mode.

        Raises
        - IllegalArgumentException: if `x <= 0`
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `x`
            is not a power of two
        """
        ...


    @staticmethod
    def log10(x: int, mode: "RoundingMode") -> int:
        """
        Returns the base-10 logarithm of `x`, rounded according to the specified rounding mode.

        Raises
        - IllegalArgumentException: if `x <= 0`
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `x`
            is not a power of ten
        """
        ...


    @staticmethod
    def pow(b: int, k: int) -> int:
        """
        Returns `b` to the `k`th power. Even if the result overflows, it will be equal to
        `BigInteger.valueOf(b).pow(k).longValue()`. This implementation runs in `O(log k)`
        time.

        Raises
        - IllegalArgumentException: if `k < 0`
        """
        ...


    @staticmethod
    def sqrt(x: int, mode: "RoundingMode") -> int:
        """
        Returns the square root of `x`, rounded with the specified rounding mode.

        Raises
        - IllegalArgumentException: if `x < 0`
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `sqrt(x)` is not an integer
        """
        ...


    @staticmethod
    def divide(p: int, q: int, mode: "RoundingMode") -> int:
        """
        Returns the result of dividing `p` by `q`, rounding using the specified `RoundingMode`.

        Raises
        - ArithmeticException: if `q == 0`, or if `mode == UNNECESSARY` and `a`
            is not an integer multiple of `b`
        """
        ...


    @staticmethod
    def mod(x: int, m: int) -> int:
        """
        Returns `x mod m`, a non-negative value less than `m`. This differs from `x %
        m`, which might be negative.
        
        For example:
        
        ````mod(7, 4) == 3
        mod(-7, 4) == 1
        mod(-1, 4) == 3
        mod(-8, 4) == 0
        mod(8, 4) == 0````

        Raises
        - ArithmeticException: if `m <= 0`

        See
        - <a href="http://docs.oracle.com/javase/specs/jls/se7/html/jls-15.html.jls-15.17.3">
            Remainder Operator</a>
        """
        ...


    @staticmethod
    def mod(x: int, m: int) -> int:
        """
        Returns `x mod m`, a non-negative value less than `m`. This differs from `x %
        m`, which might be negative.
        
        For example:
        
        ````mod(7, 4) == 3
        mod(-7, 4) == 1
        mod(-1, 4) == 3
        mod(-8, 4) == 0
        mod(8, 4) == 0````

        Raises
        - ArithmeticException: if `m <= 0`

        See
        - <a href="http://docs.oracle.com/javase/specs/jls/se7/html/jls-15.html.jls-15.17.3">
            Remainder Operator</a>
        """
        ...


    @staticmethod
    def gcd(a: int, b: int) -> int:
        """
        Returns the greatest common divisor of `a, b`. Returns `0` if `a == 0 && b ==
        0`.

        Raises
        - IllegalArgumentException: if `a < 0` or `b < 0`
        """
        ...


    @staticmethod
    def checkedAdd(a: int, b: int) -> int:
        ...


    @staticmethod
    def checkedSubtract(a: int, b: int) -> int:
        """
        Returns the difference of `a` and `b`, provided it does not overflow.

        Raises
        - ArithmeticException: if `a - b` overflows in signed `long` arithmetic
        """
        ...


    @staticmethod
    def checkedMultiply(a: int, b: int) -> int:
        ...


    @staticmethod
    def checkedPow(b: int, k: int) -> int:
        """
        Returns the `b` to the `k`th power, provided it does not overflow.

        Raises
        - ArithmeticException: if `b` to the `k`th power overflows in signed `long` arithmetic
        """
        ...


    @staticmethod
    def saturatedAdd(a: int, b: int) -> int:
        ...


    @staticmethod
    def saturatedSubtract(a: int, b: int) -> int:
        ...


    @staticmethod
    def saturatedMultiply(a: int, b: int) -> int:
        ...


    @staticmethod
    def saturatedPow(b: int, k: int) -> int:
        ...


    @staticmethod
    def factorial(n: int) -> int:
        """
        Returns `n!`, that is, the product of the first `n` positive integers, `1` if
        `n == 0`, or Long.MAX_VALUE if the result does not fit in a `long`.

        Raises
        - IllegalArgumentException: if `n < 0`
        """
        ...


    @staticmethod
    def binomial(n: int, k: int) -> int:
        """
        Returns `n` choose `k`, also known as the binomial coefficient of `n` and
        `k`, or Long.MAX_VALUE if the result does not fit in a `long`.

        Raises
        - IllegalArgumentException: if `n < 0`, `k < 0`, or `k > n`
        """
        ...


    @staticmethod
    def mean(x: int, y: int) -> int:
        """
        Returns the arithmetic mean of `x` and `y`, rounded toward negative infinity. This
        method is resilient to overflow.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def isPrime(n: int) -> bool:
        """
        Returns `True` if `n` is a <a
        href="http://mathworld.wolfram.com/PrimeNumber.html">prime number</a>: an integer *greater
        than one* that cannot be factored into a product of *smaller* positive integers.
        Returns `False` if `n` is zero, one, or a composite number (one which *can* be
        factored into smaller positive integers).
        
        To test larger numbers, use BigInteger.isProbablePrime.

        Raises
        - IllegalArgumentException: if `n` is negative

        Since
        - 20.0
        """
        ...


    @staticmethod
    def roundToDouble(x: int, mode: "RoundingMode") -> float:
        """
        Returns `x`, rounded to a `double` with the specified rounding mode. If `x`
        is precisely representable as a `double`, its `double` value will be returned;
        otherwise, the rounding will choose between the two nearest representable values with `mode`.
        
        For the case of RoundingMode.HALF_EVEN, this implementation uses the IEEE 754
        default rounding mode: if the two nearest representable values are equally near, the one with
        the least significant bit zero is chosen. (In such cases, both of the nearest representable
        values are even integers; this method returns the one that is a multiple of a greater power of
        two.)

        Raises
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `x`
            is not precisely representable as a `double`

        Since
        - 30.0
        """
        ...



"""
Python module generated from Java source file java.nio.charset.Charset

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.charset import *
from java.nio.charset.spi import CharsetProvider
from java.security import AccessController
from java.security import PrivilegedAction
from java.util import Arrays
from java.util import Collections
from java.util import Iterator
from java.util import Locale
from java.util import NoSuchElementException
from java.util import Objects
from java.util import ServiceConfigurationError
from java.util import ServiceLoader
from java.util import SortedMap
from jdk.internal.misc import VM
from sun.nio.cs import ThreadLocalCoders
from sun.security.action import GetPropertyAction
from typing import Any, Callable, Iterable, Tuple


class Charset(Comparable):

    @staticmethod
    def isSupported(charsetName: str) -> bool:
        """
        Tells whether the named charset is supported.

        Arguments
        - charsetName: The name of the requested charset; may be either
                a canonical name or an alias

        Returns
        - `True` if, and only if, support for the named charset
                 is available in the current Java virtual machine

        Raises
        - IllegalCharsetNameException: If the given charset name is illegal
        - IllegalArgumentException: If the given `charsetName` is null
        """
        ...


    @staticmethod
    def forName(charsetName: str) -> "Charset":
        """
        Returns a charset object for the named charset.

        Arguments
        - charsetName: The name of the requested charset; may be either
                a canonical name or an alias

        Returns
        - A charset object for the named charset

        Raises
        - IllegalCharsetNameException: If the given charset name is illegal
        - IllegalArgumentException: If the given `charsetName` is null
        - UnsupportedCharsetException: If no support for the named charset is available
                 in this instance of the Java virtual machine
        """
        ...


    @staticmethod
    def availableCharsets() -> "SortedMap"[str, "Charset"]:
        """
        Constructs a sorted map from canonical charset names to charset objects.
        
         The map returned by this method will have one entry for each charset
        for which support is available in the current Java virtual machine.  If
        two or more supported charsets have the same canonical name then the
        resulting map will contain just one of them; which one it will contain
        is not specified. 
        
         The invocation of this method, and the subsequent use of the
        resulting map, may cause time-consuming disk or network I/O operations
        to occur.  This method is provided for applications that need to
        enumerate all of the available charsets, for example to allow user
        charset selection.  This method is not used by the .forName
        forName method, which instead employs an efficient incremental lookup
        algorithm.
        
         This method may return different results at different times if new
        charset providers are dynamically made available to the current Java
        virtual machine.  In the absence of such changes, the charsets returned
        by this method are exactly those that can be retrieved via the .forName forName method.  

        Returns
        - An immutable, case-insensitive map from canonical charset names
                to charset objects
        """
        ...


    @staticmethod
    def defaultCharset() -> "Charset":
        """
        Returns the default charset of this Java virtual machine.
        
         The default charset is determined during virtual-machine startup and
        typically depends upon the locale and charset of the underlying
        operating system.

        Returns
        - A charset object for the default charset

        Since
        - 1.5
        """
        ...


    def name(self) -> str:
        """
        Returns this charset's canonical name.

        Returns
        - The canonical name of this charset
        """
        ...


    def aliases(self) -> set[str]:
        """
        Returns a set containing this charset's aliases.

        Returns
        - An immutable set of this charset's aliases
        """
        ...


    def displayName(self) -> str:
        """
        Returns this charset's human-readable name for the default locale.
        
         The default implementation of this method simply returns this
        charset's canonical name.  Concrete subclasses of this class may
        override this method in order to provide a localized display name. 

        Returns
        - The display name of this charset in the default locale
        """
        ...


    def isRegistered(self) -> bool:
        """
        Tells whether or not this charset is registered in the <a
        href="http://www.iana.org/assignments/character-sets">IANA Charset
        Registry</a>.

        Returns
        - `True` if, and only if, this charset is known by its
                 implementor to be registered with the IANA
        """
        ...


    def displayName(self, locale: "Locale") -> str:
        """
        Returns this charset's human-readable name for the given locale.
        
         The default implementation of this method simply returns this
        charset's canonical name.  Concrete subclasses of this class may
        override this method in order to provide a localized display name. 

        Arguments
        - locale: The locale for which the display name is to be retrieved

        Returns
        - The display name of this charset in the given locale
        """
        ...


    def contains(self, cs: "Charset") -> bool:
        """
        Tells whether or not this charset contains the given charset.
        
         A charset *C* is said to *contain* a charset *D* if,
        and only if, every character representable in *D* is also
        representable in *C*.  If this relationship holds then it is
        guaranteed that every string that can be encoded in *D* can also be
        encoded in *C* without performing any replacements.
        
         That *C* contains *D* does not imply that each character
        representable in *C* by a particular byte sequence is represented
        in *D* by the same byte sequence, although sometimes this is the
        case.
        
         Every charset contains itself.
        
         This method computes an approximation of the containment relation:
        If it returns `True` then the given charset is known to be
        contained by this charset; if it returns `False`, however, then
        it is not necessarily the case that the given charset is not contained
        in this charset.

        Arguments
        - cs: The given charset

        Returns
        - `True` if the given charset is contained in this charset
        """
        ...


    def newDecoder(self) -> "CharsetDecoder":
        """
        Constructs a new decoder for this charset.

        Returns
        - A new decoder for this charset
        """
        ...


    def newEncoder(self) -> "CharsetEncoder":
        """
        Constructs a new encoder for this charset.

        Returns
        - A new encoder for this charset

        Raises
        - UnsupportedOperationException: If this charset does not support encoding
        """
        ...


    def canEncode(self) -> bool:
        """
        Tells whether or not this charset supports encoding.
        
         Nearly all charsets support encoding.  The primary exceptions are
        special-purpose *auto-detect* charsets whose decoders can determine
        which of several possible encoding schemes is in use by examining the
        input byte sequence.  Such charsets do not support encoding because
        there is no way to determine which encoding should be used on output.
        Implementations of such charsets should override this method to return
        `False`. 

        Returns
        - `True` if, and only if, this charset supports encoding
        """
        ...


    def decode(self, bb: "ByteBuffer") -> "CharBuffer":
        """
        Convenience method that decodes bytes in this charset into Unicode
        characters.
        
         An invocation of this method upon a charset `cs` returns the
        same result as the expression
        
        ```
            cs.newDecoder()
              .onMalformedInput(CodingErrorAction.REPLACE)
              .onUnmappableCharacter(CodingErrorAction.REPLACE)
              .decode(bb); ```
        
        except that it is potentially more efficient because it can cache
        decoders between successive invocations.
        
         This method always replaces malformed-input and unmappable-character
        sequences with this charset's default replacement byte array.  In order
        to detect such sequences, use the CharsetDecoder.decode(java.nio.ByteBuffer) method directly.  

        Arguments
        - bb: The byte buffer to be decoded

        Returns
        - A char buffer containing the decoded characters
        """
        ...


    def encode(self, cb: "CharBuffer") -> "ByteBuffer":
        """
        Convenience method that encodes Unicode characters into bytes in this
        charset.
        
         An invocation of this method upon a charset `cs` returns the
        same result as the expression
        
        ```
            cs.newEncoder()
              .onMalformedInput(CodingErrorAction.REPLACE)
              .onUnmappableCharacter(CodingErrorAction.REPLACE)
              .encode(bb); ```
        
        except that it is potentially more efficient because it can cache
        encoders between successive invocations.
        
         This method always replaces malformed-input and unmappable-character
        sequences with this charset's default replacement string.  In order to
        detect such sequences, use the CharsetEncoder.encode(java.nio.CharBuffer) method directly.  

        Arguments
        - cb: The char buffer to be encoded

        Returns
        - A byte buffer containing the encoded characters
        """
        ...


    def encode(self, str: str) -> "ByteBuffer":
        """
        Convenience method that encodes a string into bytes in this charset.
        
         An invocation of this method upon a charset `cs` returns the
        same result as the expression
        
        ```
            cs.encode(CharBuffer.wrap(s)); ```

        Arguments
        - str: The string to be encoded

        Returns
        - A byte buffer containing the encoded characters
        """
        ...


    def compareTo(self, that: "Charset") -> int:
        """
        Compares this charset to another.
        
         Charsets are ordered by their canonical names, without regard to
        case. 

        Arguments
        - that: The charset to which this charset is to be compared

        Returns
        - A negative integer, zero, or a positive integer as this charset
                is less than, equal to, or greater than the specified charset
        """
        ...


    def hashCode(self) -> int:
        """
        Computes a hashcode for this charset.

        Returns
        - An integer hashcode
        """
        ...


    def equals(self, ob: "Object") -> bool:
        """
        Tells whether or not this object is equal to another.
        
         Two charsets are equal if, and only if, they have the same canonical
        names.  A charset is never equal to any other type of object.  

        Returns
        - `True` if, and only if, this charset is equal to the
                 given object
        """
        ...


    def toString(self) -> str:
        """
        Returns a string describing this charset.

        Returns
        - A string describing this charset
        """
        ...



"""
Python module generated from Java source file com.google.common.escape.UnicodeEscaper

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.escape import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class UnicodeEscaper(Escaper):
    """
    An Escaper that converts literal text into a format safe for inclusion in a particular
    context (such as an XML document). Typically (but not always), the inverse process of
    "unescaping" the text is performed automatically by the relevant parser.
    
    For example, an XML escaper would convert the literal string `"Foo<Bar>"` into `"Foo&lt;Bar&gt;"` to prevent `"<Bar>"` from being confused with an XML tag. When the
    resulting XML document is parsed, the parser API will return this text as the original literal
    string `"Foo<Bar>"`.
    
    **Note:** This class is similar to CharEscaper but with one very important
    difference. A CharEscaper can only process Java <a
    href="http://en.wikipedia.org/wiki/UTF-16">UTF16</a> characters in isolation and may not cope
    when it encounters surrogate pairs. This class facilitates the correct escaping of all Unicode
    characters.
    
    As there are important reasons, including potential security issues, to handle Unicode
    correctly if you are considering implementing a new escaper you should favor using UnicodeEscaper
    wherever possible.
    
    A `UnicodeEscaper` instance is required to be stateless, and safe when used concurrently
    by multiple threads.
    
    Popular escapers are defined as constants in classes like com.google.common.html.HtmlEscapers and com.google.common.xml.XmlEscapers. To create
    your own escapers extend this class and implement the .escape(int) method.

    Author(s)
    - David Beaumont

    Since
    - 15.0
    """

    def escape(self, string: str) -> str:
        """
        Returns the escaped form of a given literal string.
        
        If you are escaping input in arbitrary successive chunks, then it is not generally safe to
        use this method. If an input string ends with an unmatched high surrogate character, then this
        method will throw IllegalArgumentException. You should ensure your input is valid <a
        href="http://en.wikipedia.org/wiki/UTF-16">UTF-16</a> before calling this method.
        
        **Note:** When implementing an escaper it is a good idea to override this method for
        efficiency by inlining the implementation of .nextEscapeIndex(CharSequence, int, int)
        directly. Doing this for com.google.common.net.PercentEscaper more than doubled the
        performance for unescaped strings (as measured by `CharEscapersBenchmark`).

        Arguments
        - string: the literal string to be escaped

        Returns
        - the escaped form of `string`

        Raises
        - NullPointerException: if `string` is null
        - IllegalArgumentException: if invalid surrogate characters are encountered
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.CompactHashSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Objects
from com.google.common.base import Preconditions
from com.google.common.collect import *
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import IOException
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.util import AbstractSet
from java.util import Arrays
from java.util import Collections
from java.util import ConcurrentModificationException
from java.util import Iterator
from java.util import LinkedHashSet
from java.util import NoSuchElementException
from java.util import Spliterator
from java.util.function import Consumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class CompactHashSet(AbstractSet, Serializable):
    """
    CompactHashSet is an implementation of a Set. All optional operations (adding and removing) are
    supported. The elements can be any objects.
    
    `contains(x)`, `add(x)` and `remove(x)`, are all (expected and amortized)
    constant time operations. Expected in the hashtable sense (depends on the hash function doing a
    good job of distributing the elements to the buckets to a distribution not far from uniform), and
    amortized since some operations can trigger a hash table resize.
    
    Unlike `java.util.HashSet`, iteration is only proportional to the actual `size()`,
    which is optimal, and *not* the size of the internal hashtable, which could be much larger
    than `size()`. Furthermore, this structure only depends on a fixed number of arrays; `add(x)` operations *do not* create objects for the garbage collector to deal with, and for
    every element added, the garbage collector will have to traverse `1.5` references on
    average, in the marking phase, not `5.0` as in `java.util.HashSet`.
    
    If there are no removals, then .iterator iteration order is the same as insertion
    order. Any removal invalidates any ordering guarantees.
    
    This class should not be assumed to be universally superior to `java.util.HashSet`.
    Generally speaking, this class reduces object allocation and memory consumption at the price of
    moderately increased constant factors of CPU. Only use this class when there is a specific reason
    to prioritize memory over CPU.

    Author(s)
    - Jon Noack
    """

    @staticmethod
    def create() -> "CompactHashSet"["E"]:
        """
        Creates an empty `CompactHashSet` instance.
        """
        ...


    @staticmethod
    def create(collection: Iterable["E"]) -> "CompactHashSet"["E"]:
        """
        Creates a *mutable* `CompactHashSet` instance containing the elements of the given
        collection in unspecified order.

        Arguments
        - collection: the elements that the set should contain

        Returns
        - a new `CompactHashSet` containing those elements (minus duplicates)
        """
        ...


    @staticmethod
    def create(*elements: Tuple["E", ...]) -> "CompactHashSet"["E"]:
        """
        Creates a *mutable* `CompactHashSet` instance containing the given elements in
        unspecified order.

        Arguments
        - elements: the elements that the set should contain

        Returns
        - a new `CompactHashSet` containing those elements (minus duplicates)
        """
        ...


    @staticmethod
    def createWithExpectedSize(expectedSize: int) -> "CompactHashSet"["E"]:
        """
        Creates a `CompactHashSet` instance, with a high enough "initial capacity" that it
        *should* hold `expectedSize` elements without growth.

        Arguments
        - expectedSize: the number of elements you expect to add to the returned set

        Returns
        - a new, empty `CompactHashSet` with enough capacity to hold `expectedSize`
            elements without resizing

        Raises
        - IllegalArgumentException: if `expectedSize` is negative
        """
        ...


    def add(self, object: "E") -> bool:
        ...


    def contains(self, object: "Object") -> bool:
        ...


    def remove(self, object: "Object") -> bool:
        ...


    def iterator(self) -> Iterator["E"]:
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        ...


    def size(self) -> int:
        ...


    def isEmpty(self) -> bool:
        ...


    def toArray(self) -> list["Object"]:
        ...


    def toArray(self, a: list["T"]) -> list["T"]:
        ...


    def trimToSize(self) -> None:
        """
        Ensures that this `CompactHashSet` has the smallest representation in memory, given its
        current size.
        """
        ...


    def clear(self) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerRiptideEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.inventory import ItemStack
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class PlayerRiptideEvent(PlayerEvent):
    """
    This event is fired when the player activates the riptide enchantment, using
    their trident to propel them through the air.
    
    N.B. the riptide action is currently performed client side, so manipulating
    the player in this event may have undesired effects.
    """

    def __init__(self, who: "Player", item: "ItemStack", velocity: "Vector"):
        ...


    def __init__(self, who: "Player", item: "ItemStack"):
        ...


    def getItem(self) -> "ItemStack":
        """
        Gets the item containing the used enchantment.

        Returns
        - held enchanted item
        """
        ...


    def getVelocity(self) -> "Vector":
        """
        Get the velocity applied to the player as a result of this riptide.

        Returns
        - the riptide velocity
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerRecipeBookClickEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.inventory import CraftingRecipe
from org.bukkit.inventory import Recipe
from typing import Any, Callable, Iterable, Tuple


class PlayerRecipeBookClickEvent(PlayerEvent):
    """
    Called when a player clicks a recipe in the recipe book.
    """

    def __init__(self, player: "Player", recipe: "Recipe", shiftClick: bool):
        ...


    def getOriginalRecipe(self) -> "Recipe":
        """
        Gets the original recipe the player was trying to craft. 
        This *will not* reflect any changes made with setRecipe.

        Returns
        - the original recipe
        """
        ...


    def getRecipe(self) -> "Recipe":
        """
        Gets the recipe the player is trying to craft. 
        This *will* reflect changes made with setRecipe.

        Returns
        - the recipe
        """
        ...


    def setRecipe(self, recipe: "Recipe") -> None:
        """
        Set the recipe that will be used. 
        The game will attempt to move the ingredients for this recipe into the
        appropriate slots.
        
        If the original recipe is a CraftingRecipe the provided recipe
        must also be a CraftingRecipe, otherwise the provided recipe must
        be of the same type as the original recipe.

        Arguments
        - recipe: the recipe to be used
        """
        ...


    def isShiftClick(self) -> bool:
        """
        If True the game will attempt to move the ingredients for as many copies
        of this recipe as possible into the appropriate slots, otherwise only 1
        copy will be moved.

        Returns
        - whether as many copies as possible should be moved
        """
        ...


    def setShiftClick(self, shiftClick: bool) -> None:
        """
        Sets if the game will attempt to move the ingredients for as many copies
        of this recipe as possible into the appropriate slots.

        Arguments
        - shiftClick: whether as many copies as possible should be moved
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file java.text.DateFormat

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import InvalidObjectException
from java.text import *
from java.text.spi import DateFormatProvider
from java.util import Calendar
from java.util import Date
from java.util import GregorianCalendar
from java.util import Locale
from java.util import MissingResourceException
from java.util import ResourceBundle
from java.util import TimeZone
from java.util.spi import LocaleServiceProvider
from sun.util.locale.provider import LocaleProviderAdapter
from sun.util.locale.provider import LocaleServiceProviderPool
from typing import Any, Callable, Iterable, Tuple


class DateFormat(Format):
    """
    `DateFormat` is an abstract class for date/time formatting subclasses which
    formats and parses dates or time in a language-independent manner.
    The date/time formatting subclass, such as SimpleDateFormat, allows for
    formatting (i.e., date &rarr; text), parsing (text &rarr; date), and
    normalization.  The date is represented as a `Date` object or
    as the milliseconds since January 1, 1970, 00:00:00 GMT.
    
    `DateFormat` provides many class methods for obtaining default date/time
    formatters based on the default or a given locale and a number of formatting
    styles. The formatting styles include .FULL, .LONG, .MEDIUM, and .SHORT. More
    detail and examples of using these styles are provided in the method
    descriptions.
    
    `DateFormat` helps you to format and parse dates for any locale.
    Your code can be completely independent of the locale conventions for
    months, days of the week, or even the calendar format: lunar vs. solar.
    
    To format a date for the current Locale, use one of the
    static factory methods:
    <blockquote>
    ````myString = DateFormat.getDateInstance().format(myDate);````
    </blockquote>
    If you are formatting multiple dates, it is
    more efficient to get the format and use it multiple times so that
    the system doesn't have to fetch the information about the local
    language and country conventions multiple times.
    <blockquote>
    ````DateFormat df = DateFormat.getDateInstance();
    for (int i = 0; i < myDate.length; ++i) {
        output.println(df.format(myDate[i]) + "; ");`
    }```
    </blockquote>
    To format a date for a different Locale, specify it in the
    call to .getDateInstance(int, Locale) getDateInstance().
    <blockquote>
    ````DateFormat df = DateFormat.getDateInstance(DateFormat.LONG, Locale.FRANCE);````
    </blockquote>
    
    If the specified locale contains "ca" (calendar), "rg" (region override),
    and/or "tz" (timezone) <a href="../util/Locale.html#def_locale_extension">Unicode
    extensions</a>, the calendar, the country and/or the time zone for formatting
    are overridden. If both "ca" and "rg" are specified, the calendar from the "ca"
    extension supersedes the implicit one from the "rg" extension.
    
    You can use a DateFormat to parse also.
    <blockquote>
    ````myDate = df.parse(myString);````
    </blockquote>
    Use `getDateInstance` to get the normal date format for that country.
    There are other static factory methods available.
    Use `getTimeInstance` to get the time format for that country.
    Use `getDateTimeInstance` to get a date and time format. You can pass in
    different options to these factory methods to control the length of the
    result; from .SHORT to .MEDIUM to .LONG to .FULL. The exact result depends
    on the locale, but generally:
    - .SHORT is completely numeric, such as `12.13.52` or `3:30pm`
    - .MEDIUM is longer, such as `Jan 12, 1952`
    - .LONG is longer, such as `January 12, 1952` or `3:30:32pm`
    - .FULL is pretty completely specified, such as
    `Tuesday, April 12, 1952 AD or 3:30:42pm PST`.
    
    
    You can also set the time zone on the format if you wish.
    If you want even more control over the format or parsing,
    (or want to give your users more control),
    you can try casting the `DateFormat` you get from the factory methods
    to a SimpleDateFormat. This will work for the majority
    of countries; just remember to put it in a `try` block in case you
    encounter an unusual one.
    
    You can also use forms of the parse and format methods with
    ParsePosition and FieldPosition to
    allow you to
    - progressively parse through pieces of a string.
    - align any particular field, or find out where it is for selection
    on the screen.
    
    
    <h2><a id="synchronization">Synchronization</a></h2>
    
    
    Date formats are not synchronized.
    It is recommended to create separate format instances for each thread.
    If multiple threads access a format concurrently, it must be synchronized
    externally.

    Author(s)
    - Mark Davis, Chen-Lieh Huang, Alan Liu

    See
    - java.time.format.DateTimeFormatter

    Since
    - 1.1

    Unknown Tags
    - Consider using java.time.format.DateTimeFormatter as an
    immutable and thread-safe alternative.
    - - The .format(Date, StringBuffer, FieldPosition) and
    .parse(String, ParsePosition) methods may throw
    `NullPointerException`, if any of their parameter is `null`.
    The subclass may provide its own implementation and specification about
    `NullPointerException`.
    - The .setCalendar(Calendar), .setNumberFormat(NumberFormat) and .setTimeZone(TimeZone) methods
    do not throw `NullPointerException` when their parameter is
    `null`, but any subsequent operations on the same instance may throw
    `NullPointerException`.
    - The .getCalendar(), .getNumberFormat() and
    getTimeZone() methods may return `null`, if the respective
    values of this instance is set to `null` through the corresponding
    setter methods. For Example: .getTimeZone() may return `null`,
    if the `TimeZone` value of this instance is set as
    .setTimeZone(java.util.TimeZone) setTimeZone(null).
    """

    ERA_FIELD = 0
    """
    Useful constant for ERA field alignment.
    Used in FieldPosition of date/time formatting.
    """
    YEAR_FIELD = 1
    """
    Useful constant for YEAR field alignment.
    Used in FieldPosition of date/time formatting.
    """
    MONTH_FIELD = 2
    """
    Useful constant for MONTH field alignment.
    Used in FieldPosition of date/time formatting.
    """
    DATE_FIELD = 3
    """
    Useful constant for DATE field alignment.
    Used in FieldPosition of date/time formatting.
    """
    HOUR_OF_DAY1_FIELD = 4
    """
    Useful constant for one-based HOUR_OF_DAY field alignment.
    Used in FieldPosition of date/time formatting.
    HOUR_OF_DAY1_FIELD is used for the one-based 24-hour clock.
    For example, 23:59 + 01:00 results in 24:59.
    """
    HOUR_OF_DAY0_FIELD = 5
    """
    Useful constant for zero-based HOUR_OF_DAY field alignment.
    Used in FieldPosition of date/time formatting.
    HOUR_OF_DAY0_FIELD is used for the zero-based 24-hour clock.
    For example, 23:59 + 01:00 results in 00:59.
    """
    MINUTE_FIELD = 6
    """
    Useful constant for MINUTE field alignment.
    Used in FieldPosition of date/time formatting.
    """
    SECOND_FIELD = 7
    """
    Useful constant for SECOND field alignment.
    Used in FieldPosition of date/time formatting.
    """
    MILLISECOND_FIELD = 8
    """
    Useful constant for MILLISECOND field alignment.
    Used in FieldPosition of date/time formatting.
    """
    DAY_OF_WEEK_FIELD = 9
    """
    Useful constant for DAY_OF_WEEK field alignment.
    Used in FieldPosition of date/time formatting.
    """
    DAY_OF_YEAR_FIELD = 10
    """
    Useful constant for DAY_OF_YEAR field alignment.
    Used in FieldPosition of date/time formatting.
    """
    DAY_OF_WEEK_IN_MONTH_FIELD = 11
    """
    Useful constant for DAY_OF_WEEK_IN_MONTH field alignment.
    Used in FieldPosition of date/time formatting.
    """
    WEEK_OF_YEAR_FIELD = 12
    """
    Useful constant for WEEK_OF_YEAR field alignment.
    Used in FieldPosition of date/time formatting.
    """
    WEEK_OF_MONTH_FIELD = 13
    """
    Useful constant for WEEK_OF_MONTH field alignment.
    Used in FieldPosition of date/time formatting.
    """
    AM_PM_FIELD = 14
    """
    Useful constant for AM_PM field alignment.
    Used in FieldPosition of date/time formatting.
    """
    HOUR1_FIELD = 15
    """
    Useful constant for one-based HOUR field alignment.
    Used in FieldPosition of date/time formatting.
    HOUR1_FIELD is used for the one-based 12-hour clock.
    For example, 11:30 PM + 1 hour results in 12:30 AM.
    """
    HOUR0_FIELD = 16
    """
    Useful constant for zero-based HOUR field alignment.
    Used in FieldPosition of date/time formatting.
    HOUR0_FIELD is used for the zero-based 12-hour clock.
    For example, 11:30 PM + 1 hour results in 00:30 AM.
    """
    TIMEZONE_FIELD = 17
    """
    Useful constant for TIMEZONE field alignment.
    Used in FieldPosition of date/time formatting.
    """
    FULL = 0
    """
    Constant for full style pattern.
    """
    LONG = 1
    """
    Constant for long style pattern.
    """
    MEDIUM = 2
    """
    Constant for medium style pattern.
    """
    SHORT = 3
    """
    Constant for short style pattern.
    """
    DEFAULT = MEDIUM
    """
    Constant for default style pattern.  Its value is MEDIUM.
    """


    def format(self, obj: "Object", toAppendTo: "StringBuffer", fieldPosition: "FieldPosition") -> "StringBuffer":
        """
        Formats the given `Object` into a date-time string. The formatted
        string is appended to the given `StringBuffer`.

        Arguments
        - obj: Must be a `Date` or a `Number` representing a
        millisecond offset from the <a href="../util/Calendar.html#Epoch">Epoch</a>.
        - toAppendTo: The string buffer for the returning date-time string.
        - fieldPosition: keeps track on the position of the field within
        the returned string. For example, given a date-time text
        `"1996.07.10 AD at 15:08:56 PDT"`, if the given `fieldPosition`
        is DateFormat.YEAR_FIELD, the begin index and end index of
        `fieldPosition` will be set to 0 and 4, respectively.
        Notice that if the same date-time field appears more than once in a
        pattern, the `fieldPosition` will be set for the first occurrence
        of that date-time field. For instance, formatting a `Date` to the
        date-time string `"1 PM PDT (Pacific Daylight Time)"` using the
        pattern `"h a z (zzzz)"` and the alignment field
        DateFormat.TIMEZONE_FIELD, the begin index and end index of
        `fieldPosition` will be set to 5 and 8, respectively, for the
        first occurrence of the timezone pattern character `'z'`.

        Returns
        - the string buffer passed in as `toAppendTo`,
                with formatted text appended.

        Raises
        - IllegalArgumentException: if the `Format` cannot format
                   the given `obj`.

        See
        - java.text.Format
        """
        ...


    def format(self, date: "Date", toAppendTo: "StringBuffer", fieldPosition: "FieldPosition") -> "StringBuffer":
        """
        Formats a Date into a date-time string. The formatted
        string is appended to the given `StringBuffer`.

        Arguments
        - date: a Date to be formatted into a date-time string.
        - toAppendTo: the string buffer for the returning date-time string.
        - fieldPosition: keeps track on the position of the field within
        the returned string. For example, given a date-time text
        `"1996.07.10 AD at 15:08:56 PDT"`, if the given `fieldPosition`
        is DateFormat.YEAR_FIELD, the begin index and end index of
        `fieldPosition` will be set to 0 and 4, respectively.
        Notice that if the same date-time field appears more than once in a
        pattern, the `fieldPosition` will be set for the first occurrence
        of that date-time field. For instance, formatting a `Date` to the
        date-time string `"1 PM PDT (Pacific Daylight Time)"` using the
        pattern `"h a z (zzzz)"` and the alignment field
        DateFormat.TIMEZONE_FIELD, the begin index and end index of
        `fieldPosition` will be set to 5 and 8, respectively, for the
        first occurrence of the timezone pattern character `'z'`.

        Returns
        - the string buffer passed in as `toAppendTo`, with formatted
        text appended.
        """
        ...


    def format(self, date: "Date") -> str:
        """
        Formats a Date into a date-time string.

        Arguments
        - date: the time value to be formatted into a date-time string.

        Returns
        - the formatted date-time string.
        """
        ...


    def parse(self, source: str) -> "Date":
        """
        Parses text from the beginning of the given string to produce a date.
        The method may not use the entire text of the given string.
        
        See the .parse(String, ParsePosition) method for more information
        on date parsing.

        Arguments
        - source: A `String` whose beginning should be parsed.

        Returns
        - A `Date` parsed from the string.

        Raises
        - ParseException: if the beginning of the specified string
                   cannot be parsed.
        """
        ...


    def parse(self, source: str, pos: "ParsePosition") -> "Date":
        """
        Parse a date/time string according to the given parse position.  For
        example, a time text `"07/10/96 4:5 PM, PDT"` will be parsed into a `Date`
        that is equivalent to `Date(837039900000L)`.
        
         By default, parsing is lenient: If the input is not in the form used
        by this object's format method but can still be parsed as a date, then
        the parse succeeds.  Clients may insist on strict adherence to the
        format by calling .setLenient(boolean) setLenient(False).
        
        This parsing operation uses the .calendar to produce
        a `Date`. As a result, the `calendar`'s date-time
        fields and the `TimeZone` value may have been
        overwritten, depending on subclass implementations. Any `TimeZone` value that has previously been set by a call to
        .setTimeZone(java.util.TimeZone) setTimeZone may need
        to be restored for further operations.

        Arguments
        - source: The date/time string to be parsed
        - pos: On input, the position at which to start parsing; on
                     output, the position at which parsing terminated, or the
                     start position if the parse failed.

        Returns
        - A `Date`, or `null` if the input could not be parsed
        """
        ...


    def parseObject(self, source: str, pos: "ParsePosition") -> "Object":
        """
        Parses text from a string to produce a `Date`.
        
        The method attempts to parse text starting at the index given by
        `pos`.
        If parsing succeeds, then the index of `pos` is updated
        to the index after the last character used (parsing does not necessarily
        use all characters up to the end of the string), and the parsed
        date is returned. The updated `pos` can be used to
        indicate the starting point for the next call to this method.
        If an error occurs, then the index of `pos` is not
        changed, the error index of `pos` is set to the index of
        the character where the error occurred, and null is returned.
        
        See the .parse(String, ParsePosition) method for more information
        on date parsing.

        Arguments
        - source: A `String`, part of which should be parsed.
        - pos: A `ParsePosition` object with index and error
                   index information as described above.

        Returns
        - A `Date` parsed from the string. In case of
                error, returns null.

        Raises
        - NullPointerException: if `source` or `pos` is null.
        """
        ...


    @staticmethod
    def getTimeInstance() -> "DateFormat":
        """
        Gets the time formatter with the default formatting style
        for the default java.util.Locale.Category.FORMAT FORMAT locale.
        This is equivalent to calling
        .getTimeInstance(int, Locale) getTimeInstance(DEFAULT,
            Locale.getDefault(Locale.Category.FORMAT)).

        Returns
        - a time formatter.

        See
        - java.util.Locale.Category.FORMAT
        """
        ...


    @staticmethod
    def getTimeInstance(style: int) -> "DateFormat":
        """
        Gets the time formatter with the given formatting style
        for the default java.util.Locale.Category.FORMAT FORMAT locale.
        This is equivalent to calling
        .getTimeInstance(int, Locale) getTimeInstance(style,
            Locale.getDefault(Locale.Category.FORMAT)).

        Arguments
        - style: the given formatting style. For example,
        SHORT for "h:mm a" in the US locale.

        Returns
        - a time formatter.

        See
        - java.util.Locale.Category.FORMAT
        """
        ...


    @staticmethod
    def getTimeInstance(style: int, aLocale: "Locale") -> "DateFormat":
        """
        Gets the time formatter with the given formatting style
        for the given locale.

        Arguments
        - style: the given formatting style. For example,
        SHORT for "h:mm a" in the US locale.
        - aLocale: the given locale.

        Returns
        - a time formatter.
        """
        ...


    @staticmethod
    def getDateInstance() -> "DateFormat":
        """
        Gets the date formatter with the default formatting style
        for the default java.util.Locale.Category.FORMAT FORMAT locale.
        This is equivalent to calling
        .getDateInstance(int, Locale) getDateInstance(DEFAULT,
            Locale.getDefault(Locale.Category.FORMAT)).

        Returns
        - a date formatter.

        See
        - java.util.Locale.Category.FORMAT
        """
        ...


    @staticmethod
    def getDateInstance(style: int) -> "DateFormat":
        """
        Gets the date formatter with the given formatting style
        for the default java.util.Locale.Category.FORMAT FORMAT locale.
        This is equivalent to calling
        .getDateInstance(int, Locale) getDateInstance(style,
            Locale.getDefault(Locale.Category.FORMAT)).

        Arguments
        - style: the given formatting style. For example,
        SHORT for "M/d/yy" in the US locale.

        Returns
        - a date formatter.

        See
        - java.util.Locale.Category.FORMAT
        """
        ...


    @staticmethod
    def getDateInstance(style: int, aLocale: "Locale") -> "DateFormat":
        """
        Gets the date formatter with the given formatting style
        for the given locale.

        Arguments
        - style: the given formatting style. For example,
        SHORT for "M/d/yy" in the US locale.
        - aLocale: the given locale.

        Returns
        - a date formatter.
        """
        ...


    @staticmethod
    def getDateTimeInstance() -> "DateFormat":
        """
        Gets the date/time formatter with the default formatting style
        for the default java.util.Locale.Category.FORMAT FORMAT locale.
        This is equivalent to calling
        .getDateTimeInstance(int, int, Locale) getDateTimeInstance(DEFAULT,
            DEFAULT, Locale.getDefault(Locale.Category.FORMAT)).

        Returns
        - a date/time formatter.

        See
        - java.util.Locale.Category.FORMAT
        """
        ...


    @staticmethod
    def getDateTimeInstance(dateStyle: int, timeStyle: int) -> "DateFormat":
        """
        Gets the date/time formatter with the given date and time
        formatting styles for the default java.util.Locale.Category.FORMAT FORMAT locale.
        This is equivalent to calling
        .getDateTimeInstance(int, int, Locale) getDateTimeInstance(dateStyle,
            timeStyle, Locale.getDefault(Locale.Category.FORMAT)).

        Arguments
        - dateStyle: the given date formatting style. For example,
        SHORT for "M/d/yy" in the US locale.
        - timeStyle: the given time formatting style. For example,
        SHORT for "h:mm a" in the US locale.

        Returns
        - a date/time formatter.

        See
        - java.util.Locale.Category.FORMAT
        """
        ...


    @staticmethod
    def getDateTimeInstance(dateStyle: int, timeStyle: int, aLocale: "Locale") -> "DateFormat":
        """
        Gets the date/time formatter with the given formatting styles
        for the given locale.

        Arguments
        - dateStyle: the given date formatting style.
        - timeStyle: the given time formatting style.
        - aLocale: the given locale.

        Returns
        - a date/time formatter.
        """
        ...


    @staticmethod
    def getInstance() -> "DateFormat":
        """
        Get a default date/time formatter that uses the SHORT style for both the
        date and the time.

        Returns
        - a date/time formatter
        """
        ...


    @staticmethod
    def getAvailableLocales() -> list["Locale"]:
        """
        Returns an array of all locales for which the
        `get*Instance` methods of this class can return
        localized instances.
        The returned array represents the union of locales supported by the Java
        runtime and by installed
        java.text.spi.DateFormatProvider DateFormatProvider implementations.
        It must contain at least a `Locale` instance equal to
        java.util.Locale.US Locale.US.

        Returns
        - An array of locales for which localized
                `DateFormat` instances are available.
        """
        ...


    def setCalendar(self, newCalendar: "Calendar") -> None:
        """
        Set the calendar to be used by this date format.  Initially, the default
        calendar for the specified or default locale is used.
        
        Any java.util.TimeZone TimeZone and .isLenient() leniency values that have previously been set are
        overwritten by `newCalendar`'s values.

        Arguments
        - newCalendar: the new `Calendar` to be used by the date format
        """
        ...


    def getCalendar(self) -> "Calendar":
        """
        Gets the calendar associated with this date/time formatter.

        Returns
        - the calendar associated with this date/time formatter.
        """
        ...


    def setNumberFormat(self, newNumberFormat: "NumberFormat") -> None:
        """
        Allows you to set the number formatter.

        Arguments
        - newNumberFormat: the given new NumberFormat.
        """
        ...


    def getNumberFormat(self) -> "NumberFormat":
        """
        Gets the number formatter which this date/time formatter uses to
        format and parse a time.

        Returns
        - the number formatter which this date/time formatter uses.
        """
        ...


    def setTimeZone(self, zone: "TimeZone") -> None:
        """
        Sets the time zone for the calendar of this `DateFormat` object.
        This method is equivalent to the following call.
        <blockquote>````getCalendar().setTimeZone(zone)````</blockquote>
        
        The `TimeZone` set by this method is overwritten by a
        .setCalendar(java.util.Calendar) setCalendar call.
        
        The `TimeZone` set by this method may be overwritten as
        a result of a call to the parse method.

        Arguments
        - zone: the given new time zone.
        """
        ...


    def getTimeZone(self) -> "TimeZone":
        """
        Gets the time zone.
        This method is equivalent to the following call.
        <blockquote>````getCalendar().getTimeZone()````</blockquote>

        Returns
        - the time zone associated with the calendar of DateFormat.
        """
        ...


    def setLenient(self, lenient: bool) -> None:
        """
        Specify whether or not date/time parsing is to be lenient.  With
        lenient parsing, the parser may use heuristics to interpret inputs that
        do not precisely match this object's format.  With strict parsing,
        inputs must match this object's format.
        
        This method is equivalent to the following call.
        <blockquote>````getCalendar().setLenient(lenient)````</blockquote>
        
        This leniency value is overwritten by a call to .setCalendar(java.util.Calendar) setCalendar().

        Arguments
        - lenient: when `True`, parsing is lenient

        See
        - java.util.Calendar.setLenient(boolean)
        """
        ...


    def isLenient(self) -> bool:
        """
        Tell whether date/time parsing is to be lenient.
        This method is equivalent to the following call.
        <blockquote>````getCalendar().isLenient()````</blockquote>

        Returns
        - `True` if the .calendar is lenient;
                `False` otherwise.

        See
        - java.util.Calendar.isLenient()
        """
        ...


    def hashCode(self) -> int:
        """
        Overrides hashCode
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Overrides equals
        """
        ...


    def clone(self) -> "Object":
        """
        Overrides Cloneable
        """
        ...


    class Field(Field):
        """
        Defines constants that are used as attribute keys in the
        `AttributedCharacterIterator` returned
        from `DateFormat.formatToCharacterIterator` and as
        field identifiers in `FieldPosition`.
        
        The class also provides two methods to map
        between its constants and the corresponding Calendar constants.

        See
        - java.util.Calendar

        Since
        - 1.4
        """

        ERA = Field("era", Calendar.ERA)
        """
        Constant identifying the era field.
        """
        YEAR = Field("year", Calendar.YEAR)
        """
        Constant identifying the year field.
        """
        MONTH = Field("month", Calendar.MONTH)
        """
        Constant identifying the month field.
        """
        DAY_OF_MONTH = Field("day of month", Calendar.DAY_OF_MONTH)
        """
        Constant identifying the day of month field.
        """
        HOUR_OF_DAY1 = Field("hour of day 1", -1)
        """
        Constant identifying the hour of day field, where the legal values
        are 1 to 24.
        """
        HOUR_OF_DAY0 = Field("hour of day", Calendar.HOUR_OF_DAY)
        """
        Constant identifying the hour of day field, where the legal values
        are 0 to 23.
        """
        MINUTE = Field("minute", Calendar.MINUTE)
        """
        Constant identifying the minute field.
        """
        SECOND = Field("second", Calendar.SECOND)
        """
        Constant identifying the second field.
        """
        MILLISECOND = Field("millisecond", Calendar.MILLISECOND)
        """
        Constant identifying the millisecond field.
        """
        DAY_OF_WEEK = Field("day of week", Calendar.DAY_OF_WEEK)
        """
        Constant identifying the day of week field.
        """
        DAY_OF_YEAR = Field("day of year", Calendar.DAY_OF_YEAR)
        """
        Constant identifying the day of year field.
        """
        DAY_OF_WEEK_IN_MONTH = Field("day of week in month", Calendar.DAY_OF_WEEK_IN_MONTH)
        """
        Constant identifying the day of week field.
        """
        WEEK_OF_YEAR = Field("week of year", Calendar.WEEK_OF_YEAR)
        """
        Constant identifying the week of year field.
        """
        WEEK_OF_MONTH = Field("week of month", Calendar.WEEK_OF_MONTH)
        """
        Constant identifying the week of month field.
        """
        AM_PM = Field("am pm", Calendar.AM_PM)
        """
        Constant identifying the time of day indicator
        (e.g. "a.m." or "p.m.") field.
        """
        HOUR1 = Field("hour 1", -1)
        """
        Constant identifying the hour field, where the legal values are
        1 to 12.
        """
        HOUR0 = Field("hour", Calendar.HOUR)
        """
        Constant identifying the hour field, where the legal values are
        0 to 11.
        """
        TIME_ZONE = Field("time zone", -1)
        """
        Constant identifying the time zone field.
        """


        @staticmethod
        def ofCalendarField(calendarField: int) -> "Field":
            """
            Returns the `Field` constant that corresponds to
            the `Calendar` constant `calendarField`.
            If there is no direct mapping between the `Calendar`
            constant and a `Field`, null is returned.

            Arguments
            - calendarField: Calendar field constant

            Returns
            - Field instance representing calendarField.

            Raises
            - IllegalArgumentException: if `calendarField` is
                    not the value of a `Calendar` field constant.

            See
            - java.util.Calendar
            """
            ...


        def getCalendarField(self) -> int:
            """
            Returns the `Calendar` field associated with this
            attribute. For example, if this represents the hours field of
            a `Calendar`, this would return
            `Calendar.HOUR`. If there is no corresponding
            `Calendar` constant, this will return -1.

            Returns
            - Calendar constant for this field

            See
            - java.util.Calendar
            """
            ...



"""
Python module generated from Java source file org.bukkit.entity.Fireball

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class Fireball(Projectile, Explosive):
    """
    Represents a Fireball.
    """

    def setDirection(self, direction: "Vector") -> None:
        """
        Sets the direction the fireball should be flying towards.
        
        This is a convenience method, it will change the velocity direction and
        acceleration direction, while keeping the power the same.
        
        **Note:** This method only uses the direction of the vector and will
        normalize (a copy of) it.
        
        **Special Case:** When the given direction is
        Vector.isZero() zero, the velocity and acceleration will also be
        set to zero without keeping the power.

        Arguments
        - direction: the direction this fireball should be flying towards

        See
        - .setAcceleration(Vector)
        """
        ...


    def getDirection(self) -> "Vector":
        """
        Retrieve the direction this fireball is heading toward.
        The returned vector is not normalized.

        Returns
        - the direction

        See
        - .getAcceleration()

        Deprecated
        - badly named method, returns the value of
        .getAcceleration()
        """
        ...


    def setAcceleration(self, acceleration: "Vector") -> None:
        """
        Sets the acceleration of the fireball.
        
        The acceleration gets applied to the velocity every tick, depending on
        the specific type of the fireball a damping / drag factor is applied so
        that the velocity does not grow into infinity.
        
        **Note:** that the client may not respect non-default acceleration
        power and will therefore mispredict the location of the fireball, causing
        visual stutter.

        Arguments
        - acceleration: the acceleration
        """
        ...


    def getAcceleration(self) -> "Vector":
        """
        Retrieve the acceleration of this fireball.

        Returns
        - the acceleration
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ContiguousSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import DoNotCall
from java.util import Collections
from java.util import NoSuchElementException
from typing import Any, Callable, Iterable, Tuple


class ContiguousSet(ImmutableSortedSet):
    """
    A sorted set of contiguous values in a given DiscreteDomain. Example:
    
    ````ContiguousSet.create(Range.closed(5, 42), DiscreteDomain.integers())````
    
    Note that because bounded ranges over `int` and `long` values are so common, this
    particular example can be written as just:
    
    ````ContiguousSet.closed(5, 42)````
    
    **Warning:** Be extremely careful what you do with conceptually large instances (such as
    `ContiguousSet.create(Range.greaterThan(0), DiscreteDomain.integers()`). Certain operations
    on such a set can be performed efficiently, but others (such as Set.hashCode or Collections.frequency) can cause major performance problems.

    Author(s)
    - Gregory Kick

    Since
    - 10.0
    """

    @staticmethod
    def create(range: "Range"["C"], domain: "DiscreteDomain"["C"]) -> "ContiguousSet"["C"]:
        """
        Returns a `ContiguousSet` containing the same values in the given domain Range.contains contained by the range.

        Raises
        - IllegalArgumentException: if neither range nor the domain has a lower bound, or if
            neither has an upper bound

        Since
        - 13.0
        """
        ...


    @staticmethod
    def closed(lower: int, upper: int) -> "ContiguousSet"["Integer"]:
        """
        Returns a nonempty contiguous set containing all `int` values from `lower`
        (inclusive) to `upper` (inclusive). (These are the same values contained in `Range.closed(lower, upper)`.)

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`

        Since
        - 23.0
        """
        ...


    @staticmethod
    def closed(lower: int, upper: int) -> "ContiguousSet"["Long"]:
        """
        Returns a nonempty contiguous set containing all `long` values from `lower`
        (inclusive) to `upper` (inclusive). (These are the same values contained in `Range.closed(lower, upper)`.)

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`

        Since
        - 23.0
        """
        ...


    @staticmethod
    def closedOpen(lower: int, upper: int) -> "ContiguousSet"["Integer"]:
        """
        Returns a contiguous set containing all `int` values from `lower` (inclusive) to
        `upper` (exclusive). If the endpoints are equal, an empty set is returned. (These are the
        same values contained in `Range.closedOpen(lower, upper)`.)

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`

        Since
        - 23.0
        """
        ...


    @staticmethod
    def closedOpen(lower: int, upper: int) -> "ContiguousSet"["Long"]:
        """
        Returns a contiguous set containing all `long` values from `lower` (inclusive) to
        `upper` (exclusive). If the endpoints are equal, an empty set is returned. (These are the
        same values contained in `Range.closedOpen(lower, upper)`.)

        Raises
        - IllegalArgumentException: if `lower` is greater than `upper`

        Since
        - 23.0
        """
        ...


    def headSet(self, toElement: "C") -> "ContiguousSet"["C"]:
        ...


    def headSet(self, toElement: "C", inclusive: bool) -> "ContiguousSet"["C"]:
        """
        Since
        - 12.0
        """
        ...


    def subSet(self, fromElement: "C", toElement: "C") -> "ContiguousSet"["C"]:
        ...


    def subSet(self, fromElement: "C", fromInclusive: bool, toElement: "C", toInclusive: bool) -> "ContiguousSet"["C"]:
        """
        Since
        - 12.0
        """
        ...


    def tailSet(self, fromElement: "C") -> "ContiguousSet"["C"]:
        ...


    def tailSet(self, fromElement: "C", inclusive: bool) -> "ContiguousSet"["C"]:
        """
        Since
        - 12.0
        """
        ...


    def intersection(self, other: "ContiguousSet"["C"]) -> "ContiguousSet"["C"]:
        """
        Returns the set of values that are contained in both this set and the other.
        
        This method should always be used instead of Sets.intersection for ContiguousSet instances.
        """
        ...


    def range(self) -> "Range"["C"]:
        """
        Returns a range, closed on both ends, whose endpoints are the minimum and maximum values
        contained in this set. This is equivalent to `range(CLOSED, CLOSED)`.

        Raises
        - NoSuchElementException: if this set is empty
        """
        ...


    def range(self, lowerBoundType: "BoundType", upperBoundType: "BoundType") -> "Range"["C"]:
        """
        Returns the minimal range with the given boundary types for which all values in this set are
        Range.contains(Comparable) contained within the range.
        
        Note that this method will return ranges with unbounded endpoints if BoundType.OPEN
        is requested for a domain minimum or maximum. For example, if `set` was created from the
        range `[1..Integer.MAX_VALUE]` then `set.range(CLOSED, OPEN)` must return `[1..∞)`.

        Raises
        - NoSuchElementException: if this set is empty
        """
        ...


    def toString(self) -> str:
        """
        Returns a shorthand representation of the contents such as `"[1..100]"`.
        """
        ...


    @staticmethod
    def builder() -> "ImmutableSortedSet.Builder"["E"]:
        """
        Not supported. `ContiguousSet` instances are constructed with .create. This
        method exists only to hide ImmutableSet.builder from consumers of `ContiguousSet`.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Use .create.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.FallingBlock

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block.data import BlockData
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class FallingBlock(Entity):
    """
    Represents a falling block
    """

    def getMaterial(self) -> "Material":
        """
        Get the Material of the falling block

        Returns
        - Material of the block

        Deprecated
        - use .getBlockData()
        """
        ...


    def getBlockData(self) -> "BlockData":
        """
        Get the data for the falling block

        Returns
        - data of the block
        """
        ...


    def getDropItem(self) -> bool:
        """
        Get if the falling block will break into an item if it cannot be placed.
        
        Note that if .getCancelDrop() is `True`, the falling block
        will not drop an item regardless of whether or not the returned value is
        `True`.

        Returns
        - True if the block will break into an item when obstructed
        """
        ...


    def setDropItem(self, drop: bool) -> None:
        """
        Set if the falling block will break into an item if it cannot be placed.
        
        Note that if .getCancelDrop() is `True`, the falling block
        will not drop an item regardless of whether or not the value is set to
        `True`.

        Arguments
        - drop: True to break into an item when obstructed
        """
        ...


    def getCancelDrop(self) -> bool:
        """
        Get if the falling block will not become a block upon landing and not drop
        an item.
        
        Unlike .getDropItem(), this property will prevent the block from
        forming into a block when it lands, causing it to disappear. If this property
        is True and .getDropItem() is True, an item will <strong>NOT</strong>
        be dropped.

        Returns
        - True if the block will disappear
        """
        ...


    def setCancelDrop(self, cancelDrop: bool) -> None:
        """
        Get if the falling block will not become a block upon landing and not drop
        an item.
        
        Unlike .setDropItem(boolean), this property will prevent the block
        from forming into a block when it lands, causing it to disappear. If this
        property is True and .getDropItem() is True, an item will
        <strong>NOT</strong> be dropped.

        Arguments
        - cancelDrop: True to make the block disappear when landing
        """
        ...


    def canHurtEntities(self) -> bool:
        """
        Get the HurtEntities state of this block.

        Returns
        - whether entities will be damaged by this block.
        """
        ...


    def setHurtEntities(self, hurtEntities: bool) -> None:
        """
        Set the HurtEntities state of this block.

        Arguments
        - hurtEntities: whether entities will be damaged by this block.
        """
        ...


    def getDamagePerBlock(self) -> float:
        """
        Get the amount of damage inflicted upon entities multiplied by the distance
        that the block had fallen when this falling block lands on them.

        Returns
        - the damage per block
        """
        ...


    def setDamagePerBlock(self, damage: float) -> None:
        """
        Set the amount of damage inflicted upon entities multiplied by the distance
        that the block had fallen when this falling block lands on them.
        
        If `damage` is non-zero, this method will automatically call
        .setHurtEntities(boolean) setHurtEntities(True).

        Arguments
        - damage: the damage per block to set. Must be >= 0.0
        """
        ...


    def getMaxDamage(self) -> int:
        """
        Get the maximum amount of damage that can be inflicted upon entities when
        this falling block lands on them.

        Returns
        - the max damage
        """
        ...


    def setMaxDamage(self, damage: int) -> None:
        """
        Set the maximum amount of damage that can be inflicted upon entities when
        this falling block lands on them.
        
        If `damage` is non-zero, this method will automatically call
        .setHurtEntities(boolean) setHurtEntities(True).

        Arguments
        - damage: the max damage to set. Must be >= 0
        """
        ...



"""
Python module generated from Java source file com.google.common.cache.Weigher

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.cache import *
from typing import Any, Callable, Iterable, Tuple


class Weigher:
    """
    Calculates the weights of cache entries.

    Author(s)
    - Charles Fry

    Since
    - 11.0
    """

    def weigh(self, key: "K", value: "V") -> int:
        """
        Returns the weight of a cache entry. There is no unit for entry weights; rather they are simply
        relative to each other.

        Returns
        - the weight of the entry; must be non-negative
        """
        ...



"""
Python module generated from Java source file com.google.common.annotations.Beta

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import *
from typing import Any, Callable, Iterable, Tuple


class Beta:
    """
    Signifies that a public API (public class, method or field) is subject to incompatible changes,
    or even removal, in a future release. An API bearing this annotation is exempt from any
    compatibility guarantees made by its containing library. Note that the presence of this
    annotation implies nothing about the quality or performance of the API in question, only the fact
    that it is not "API-frozen."
    
    It is generally safe for *applications* to depend on beta APIs, at the cost of some extra
    work during upgrades. However it is generally inadvisable for *libraries* (which get
    included on users' CLASSPATHs, outside the library developers' control) to do so.

    Author(s)
    - Kevin Bourrillion
    """





"""
Python module generated from Java source file org.bukkit.scoreboard.Team

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import ChatColor
from org.bukkit import OfflinePlayer
from org.bukkit.potion import PotionEffectType
from org.bukkit.scoreboard import *
from typing import Any, Callable, Iterable, Tuple


class Team:
    """
    A team on a scoreboard that has a common display theme and other
    properties. This team is only relevant to the display of the associated
    .getScoreboard() scoreboard.
    """

    def getName(self) -> str:
        """
        Gets the name of this Team

        Returns
        - Objective name

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def getDisplayName(self) -> str:
        """
        Gets the name displayed to entries for this team

        Returns
        - Team display name

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def setDisplayName(self, displayName: str) -> None:
        """
        Sets the name displayed to entries for this team

        Arguments
        - displayName: New display name

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def getPrefix(self) -> str:
        """
        Gets the prefix prepended to the display of entries on this team.

        Returns
        - Team prefix

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def setPrefix(self, prefix: str) -> None:
        """
        Sets the prefix prepended to the display of entries on this team.

        Arguments
        - prefix: New prefix

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def getSuffix(self) -> str:
        """
        Gets the suffix appended to the display of entries on this team.

        Returns
        - the team's current suffix

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def setSuffix(self, suffix: str) -> None:
        """
        Sets the suffix appended to the display of entries on this team.

        Arguments
        - suffix: the new suffix for this team.

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def getColor(self) -> "ChatColor":
        """
        Gets the color of the team.
        
        This only sets the team outline, other occurrences of colors such as in
        names are handled by prefixes / suffixes.

        Returns
        - team color, defaults to ChatColor.RESET

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def setColor(self, color: "ChatColor") -> None:
        """
        Sets the color of the team.
        
        This only sets the team outline, other occurrences of colors such as in
        names are handled by prefixes / suffixes.

        Arguments
        - color: new color, must be non-null. Use ChatColor.RESET for
        no color
        """
        ...


    def allowFriendlyFire(self) -> bool:
        """
        Gets the team friendly fire state

        Returns
        - True if friendly fire is enabled

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def setAllowFriendlyFire(self, enabled: bool) -> None:
        """
        Sets the team friendly fire state

        Arguments
        - enabled: True if friendly fire is to be allowed

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def canSeeFriendlyInvisibles(self) -> bool:
        """
        Gets the team's ability to see PotionEffectType.INVISIBILITY
        invisible teammates.

        Returns
        - True if team members can see invisible members

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def setCanSeeFriendlyInvisibles(self, enabled: bool) -> None:
        """
        Sets the team's ability to see PotionEffectType.INVISIBILITY
        invisible teammates.

        Arguments
        - enabled: True if invisible teammates are to be visible

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def getNameTagVisibility(self) -> "NameTagVisibility":
        """
        Gets the team's ability to see name tags

        Returns
        - the current name tag visibility for the team

        Raises
        - IllegalArgumentException: if this team has been unregistered

        Deprecated
        - see .getOption(Team.Option)
        """
        ...


    def setNameTagVisibility(self, visibility: "NameTagVisibility") -> None:
        """
        Set's the team's ability to see name tags

        Arguments
        - visibility: The nameTagVisibility to set

        Raises
        - IllegalArgumentException: if this team has been unregistered

        Deprecated
        - see .setOption(Team.Option, Team.OptionStatus)
        """
        ...


    def getPlayers(self) -> set["OfflinePlayer"]:
        """
        Gets the Set of players on the team

        Returns
        - players on the team

        Raises
        - IllegalStateException: if this team has been unregistered

        See
        - .getEntries()

        Deprecated
        - Teams can contain entries that aren't players
        """
        ...


    def getEntries(self) -> set[str]:
        """
        Gets the Set of entries on the team

        Returns
        - entries on the team

        Raises
        - IllegalStateException: if this entries has been unregistered
        """
        ...


    def getSize(self) -> int:
        """
        Gets the size of the team

        Returns
        - number of entries on the team

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def getScoreboard(self) -> "Scoreboard":
        """
        Gets the Scoreboard to which this team is attached

        Returns
        - Owning scoreboard, or null if this team has been .unregister() unregistered
        """
        ...


    def addPlayer(self, player: "OfflinePlayer") -> None:
        """
        This puts the specified player onto this team for the scoreboard.
        
        This will remove the player from any other team on the scoreboard.

        Arguments
        - player: the player to add

        Raises
        - IllegalStateException: if this team has been unregistered

        See
        - .addEntry(String)

        Deprecated
        - Teams can contain entries that aren't players
        """
        ...


    def addEntry(self, entry: str) -> None:
        """
        This puts the specified entry onto this team for the scoreboard.
        
        This will remove the entry from any other team on the scoreboard.

        Arguments
        - entry: the entry to add

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def removePlayer(self, player: "OfflinePlayer") -> bool:
        """
        Removes the player from this team.

        Arguments
        - player: the player to remove

        Returns
        - if the player was on this team

        Raises
        - IllegalStateException: if this team has been unregistered

        See
        - .removeEntry(String)

        Deprecated
        - Teams can contain entries that aren't players
        """
        ...


    def removeEntry(self, entry: str) -> bool:
        """
        Removes the entry from this team.

        Arguments
        - entry: the entry to remove

        Returns
        - if the entry was a part of this team

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def unregister(self) -> None:
        """
        Unregisters this team from the Scoreboard

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def hasPlayer(self, player: "OfflinePlayer") -> bool:
        """
        Checks to see if the specified player is a member of this team.

        Arguments
        - player: the player to search for

        Returns
        - True if the player is a member of this team

        Raises
        - IllegalStateException: if this team has been unregistered

        See
        - .hasEntry(String)

        Deprecated
        - Teams can contain entries that aren't players
        """
        ...


    def hasEntry(self, entry: str) -> bool:
        """
        Checks to see if the specified entry is a member of this team.

        Arguments
        - entry: the entry to search for

        Returns
        - True if the entry is a member of this team

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def getOption(self, option: "Option") -> "OptionStatus":
        """
        Get an option for this team

        Arguments
        - option: the option to get

        Returns
        - the option status

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    def setOption(self, option: "Option", status: "OptionStatus") -> None:
        """
        Set an option for this team

        Arguments
        - option: the option to set
        - status: the new option status

        Raises
        - IllegalStateException: if this team has been unregistered
        """
        ...


    class Option(Enum):
        """
        Represents an option which may be applied to this team.
        """

        NAME_TAG_VISIBILITY = 0
        """
        How to display the name tags of players on this team.
        """
        DEATH_MESSAGE_VISIBILITY = 1
        """
        How to display the death messages for players on this team.
        """
        COLLISION_RULE = 2
        """
        How players of this team collide with others.
        """


    class OptionStatus(Enum):
        """
        How an option may be applied to members of this team.
        """

        ALWAYS = 0
        """
        Apply this option to everyone.
        """
        NEVER = 1
        """
        Never apply this option.
        """
        FOR_OTHER_TEAMS = 2
        """
        Apply this option only for opposing teams.
        """
        FOR_OWN_TEAM = 3
        """
        Apply this option for only team members.
        """



"""
Python module generated from Java source file org.joml.Matrix2fc

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Matrix2fc:
    """
    Interface to a read-only view of a 2x2 matrix of single-precision floats.

    Author(s)
    - Joseph Burton
    """

    def m00(self) -> float:
        """
        Return the value of the matrix element at column 0 and row 0.

        Returns
        - the value of the matrix element
        """
        ...


    def m01(self) -> float:
        """
        Return the value of the matrix element at column 0 and row 1.

        Returns
        - the value of the matrix element
        """
        ...


    def m10(self) -> float:
        """
        Return the value of the matrix element at column 1 and row 0.

        Returns
        - the value of the matrix element
        """
        ...


    def m11(self) -> float:
        """
        Return the value of the matrix element at column 1 and row 1.

        Returns
        - the value of the matrix element
        """
        ...


    def mul(self, right: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        """
        Multiply this matrix by the supplied `right` matrix and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the `right` matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        transformation of the right matrix will be applied first!

        Arguments
        - right: the right operand of the matrix multiplication
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulLocal(self, left: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        """
        Pre-multiply this matrix by the supplied `left` matrix and store the result in `dest`.
        
        If `M` is `this` matrix and `L` the `left` matrix,
        then the new matrix will be `L * M`. So when transforming a
        vector `v` with the new matrix by using `L * M * v`, the
        transformation of `this` matrix will be applied first!

        Arguments
        - left: the left operand of the matrix multiplication
        - dest: the destination matrix, which will hold the result

        Returns
        - dest
        """
        ...


    def determinant(self) -> float:
        """
        Return the determinant of this matrix.

        Returns
        - the determinant
        """
        ...


    def invert(self, dest: "Matrix2f") -> "Matrix2f":
        """
        Invert the `this` matrix and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transpose(self, dest: "Matrix2f") -> "Matrix2f":
        """
        Transpose `this` matrix and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, dest: "Matrix2f") -> "Matrix2f":
        """
        Get the current values of `this` matrix and store them into
        `dest`.

        Arguments
        - dest: the destination matrix

        Returns
        - the passed in destination
        """
        ...


    def get(self, dest: "Matrix3x2f") -> "Matrix3x2f":
        """
        Get the current values of `this` matrix and store them as
        the rotational component of `dest`. All other values of `dest` will
        be set to 0.

        Arguments
        - dest: the destination matrix

        Returns
        - the passed in destination

        See
        - Matrix3x2f.set(Matrix2fc)
        """
        ...


    def get(self, dest: "Matrix3f") -> "Matrix3f":
        """
        Get the current values of `this` matrix and store them as
        the rotational component of `dest`. All other values of `dest` will
        be set to identity.

        Arguments
        - dest: the destination matrix

        Returns
        - the passed in destination

        See
        - Matrix3f.set(Matrix2fc)
        """
        ...


    def getRotation(self) -> float:
        """
        Get the angle of the rotation component of `this` matrix.
        
        This method assumes that there is a valid rotation to be returned, i.e. that
        `atan2(-m10, m00) == atan2(m01, m11)`.

        Returns
        - the angle
        """
        ...


    def get(self, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this matrix in column-major order into the supplied FloatBuffer at the current
        buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the matrix is stored, use .get(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get(int, FloatBuffer)
        """
        ...


    def get(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this matrix in column-major order into the supplied FloatBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .get(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get(int, ByteBuffer)
        """
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def getTransposed(self, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store the transpose of this matrix in column-major order into the supplied FloatBuffer at the current
        buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the matrix is stored, use .getTransposed(int, FloatBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .getTransposed(int, FloatBuffer)
        """
        ...


    def getTransposed(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store the transpose of this matrix in column-major order into the supplied FloatBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def getTransposed(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store the transpose of this matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .getTransposed(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .getTransposed(int, ByteBuffer)
        """
        ...


    def getTransposed(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store the transpose of this matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def getToAddress(self, address: int) -> "Matrix2fc":
        """
        Store this matrix in column-major order at the given off-heap address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap address where to store this matrix

        Returns
        - this
        """
        ...


    def getTransposedToAddress(self, address: int) -> "Matrix2fc":
        """
        Store this matrix in row-major order at the given off-heap address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap address where to store this matrix

        Returns
        - this
        """
        ...


    def get(self, arr: list[float], offset: int) -> list[float]:
        """
        Store this matrix into the supplied float array in column-major order at the given offset.

        Arguments
        - arr: the array to write the matrix values into
        - offset: the offset into the array

        Returns
        - the passed in array
        """
        ...


    def get(self, arr: list[float]) -> list[float]:
        """
        Store this matrix into the supplied float array in column-major order.
        
        In order to specify an explicit offset into the array, use the method .get(float[], int).

        Arguments
        - arr: the array to write the matrix values into

        Returns
        - the passed in array

        See
        - .get(float[], int)
        """
        ...


    def scale(self, xy: "Vector2fc", dest: "Matrix2f") -> "Matrix2f":
        """
        Apply scaling to `this` matrix by scaling the base axes by the given `xy.x` and
        `xy.y` factors, respectively and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, x: float, y: float, dest: "Matrix2f") -> "Matrix2f":
        """
        Apply scaling to this matrix by scaling the base axes by the given x and
        y factors and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, xy: float, dest: "Matrix2f") -> "Matrix2f":
        """
        Apply scaling to this matrix by uniformly scaling all base axes by the given `xy` factor
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!

        Arguments
        - xy: the factor for all components
        - dest: will hold the result

        Returns
        - dest

        See
        - .scale(float, float, Matrix2f)
        """
        ...


    def scaleLocal(self, x: float, y: float, dest: "Matrix2f") -> "Matrix2f":
        """
        Pre-multiply scaling to `this` matrix by scaling the base axes by the given x and
        y factors and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`
        , the scaling will be applied last!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, v: "Vector2f") -> "Vector2f":
        """
        Transform the given vector by this matrix.

        Arguments
        - v: the vector to transform

        Returns
        - v
        """
        ...


    def transform(self, v: "Vector2fc", dest: "Vector2f") -> "Vector2f":
        """
        Transform the given vector by this matrix and store the result in `dest`.

        Arguments
        - v: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transform(self, x: float, y: float, dest: "Vector2f") -> "Vector2f":
        """
        Transform the vector `(x, y)` by this matrix and store the result in `dest`.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformTranspose(self, v: "Vector2f") -> "Vector2f":
        """
        Transform the given vector by the transpose of this matrix.

        Arguments
        - v: the vector to transform

        Returns
        - v
        """
        ...


    def transformTranspose(self, v: "Vector2fc", dest: "Vector2f") -> "Vector2f":
        """
        Transform the given vector by the transpose of this matrix and store the result in `dest`.

        Arguments
        - v: the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def transformTranspose(self, x: float, y: float, dest: "Vector2f") -> "Vector2f":
        """
        Transform the vector `(x, y)` by the transpose of this matrix and store the result in `dest`.

        Arguments
        - x: the x coordinate of the vector to transform
        - y: the y coordinate of the vector to transform
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotate(self, ang: float, dest: "Matrix2f") -> "Matrix2f":
        """
        Apply rotation to this matrix by rotating the given amount of radians
        and store the result in `dest`.
        
        The produced rotation will rotate a vector counter-clockwise around the origin.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`
        , the rotation will be applied first!
        
        Reference: <a href="https://en.wikipedia.org/wiki/Rotation_matrix#In_two_dimensions">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateLocal(self, ang: float, dest: "Matrix2f") -> "Matrix2f":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians
        and store the result in `dest`.
        
        The produced rotation will rotate a vector counter-clockwise around the origin.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        Reference: <a href="https://en.wikipedia.org/wiki/Rotation_matrix#In_two_dimensions">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def getRow(self, row: int, dest: "Vector2f") -> "Vector2f":
        """
        Get the row at the given `row` index, starting with `0`.

        Arguments
        - row: the row index in `[0..1]`
        - dest: will hold the row components

        Returns
        - the passed in destination

        Raises
        - IndexOutOfBoundsException: if `row` is not in `[0..1]`
        """
        ...


    def getColumn(self, column: int, dest: "Vector2f") -> "Vector2f":
        """
        Get the column at the given `column` index, starting with `0`.

        Arguments
        - column: the column index in `[0..1]`
        - dest: will hold the column components

        Returns
        - the passed in destination

        Raises
        - IndexOutOfBoundsException: if `column` is not in `[0..1]`
        """
        ...


    def get(self, column: int, row: int) -> float:
        """
        Get the matrix element value at the given column and row.

        Arguments
        - column: the colum index in `[0..1]`
        - row: the row index in `[0..1]`

        Returns
        - the element value
        """
        ...


    def normal(self, dest: "Matrix2f") -> "Matrix2f":
        """
        Compute a normal matrix from `this` matrix and store it into `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def getScale(self, dest: "Vector2f") -> "Vector2f":
        """
        Get the scaling factors of `this` matrix for the three base axes.

        Arguments
        - dest: will hold the scaling factors for `x` and `y`

        Returns
        - dest
        """
        ...


    def positiveX(self, dest: "Vector2f") -> "Vector2f":
        """
        Obtain the direction of `+X` before the transformation represented by `this` matrix is applied.
        
        This method is equivalent to the following code:
        ```
        Matrix2f inv = new Matrix2f(this).invert();
        inv.transform(dir.set(1, 0)).normalize();
        ```
        If `this` is already an orthogonal matrix, then consider using .normalizedPositiveX(Vector2f) instead.

        Arguments
        - dest: will hold the direction of `+X`

        Returns
        - dest
        """
        ...


    def normalizedPositiveX(self, dest: "Vector2f") -> "Vector2f":
        """
        Obtain the direction of `+X` before the transformation represented by `this` *orthogonal* matrix is applied.
        This method only produces correct results if `this` is an *orthogonal* matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix2f inv = new Matrix2f(this).transpose();
        inv.transform(dir.set(1, 0));
        ```

        Arguments
        - dest: will hold the direction of `+X`

        Returns
        - dest
        """
        ...


    def positiveY(self, dest: "Vector2f") -> "Vector2f":
        """
        Obtain the direction of `+Y` before the transformation represented by `this` matrix is applied.
        
        This method is equivalent to the following code:
        ```
        Matrix2f inv = new Matrix2f(this).invert();
        inv.transform(dir.set(0, 1)).normalize();
        ```
        If `this` is already an orthogonal matrix, then consider using .normalizedPositiveY(Vector2f) instead.

        Arguments
        - dest: will hold the direction of `+Y`

        Returns
        - dest
        """
        ...


    def normalizedPositiveY(self, dest: "Vector2f") -> "Vector2f":
        """
        Obtain the direction of `+Y` before the transformation represented by `this` *orthogonal* matrix is applied.
        This method only produces correct results if `this` is an *orthogonal* matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix2f inv = new Matrix2f(this).transpose();
        inv.transform(dir.set(0, 1));
        ```

        Arguments
        - dest: will hold the direction of `+Y`

        Returns
        - dest
        """
        ...


    def add(self, other: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        """
        Component-wise add `this` and `other` and store the result in `dest`.

        Arguments
        - other: the other addend
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def sub(self, subtrahend: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        """
        Component-wise subtract `subtrahend` from `this` and store the result in `dest`.

        Arguments
        - subtrahend: the subtrahend
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulComponentWise(self, other: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        """
        Component-wise multiply `this` by `other` and store the result in `dest`.

        Arguments
        - other: the other matrix
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def lerp(self, other: "Matrix2fc", t: float, dest: "Matrix2f") -> "Matrix2f":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `dest`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other matrix
        - t: the interpolation factor between 0.0 and 1.0
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def equals(self, m: "Matrix2fc", delta: float) -> bool:
        """
        Compare the matrix elements of `this` matrix with the given matrix using the given `delta`
        and return whether all of them are equal within a maximum difference of `delta`.
        
        Please note that this method is not used by any data structure such as ArrayList HashSet or HashMap
        and their operations, such as ArrayList.contains(Object) or HashSet.remove(Object), since those
        data structures only use the Object.equals(Object) and Object.hashCode() methods.

        Arguments
        - m: the other matrix
        - delta: the allowed maximum difference

        Returns
        - `True` whether all of the matrix elements are equal; `False` otherwise
        """
        ...


    def isFinite(self) -> bool:
        """
        Determine whether all matrix elements are finite floating-point values, that
        is, they are not Float.isNaN() NaN and not
        Float.isInfinite() infinity.

        Returns
        - `True` if all components are finite floating-point values;
                `False` otherwise
        """
        ...



"""
Python module generated from Java source file java.util.Spliterator

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from java.util.function import Consumer
from java.util.function import DoubleConsumer
from java.util.function import IntConsumer
from java.util.function import LongConsumer
from typing import Any, Callable, Iterable, Tuple


class Spliterator:
    """
    An object for traversing and partitioning elements of a source.  The source
    of elements covered by a Spliterator could be, for example, an array, a
    Collection, an IO channel, or a generator function.
    
    A Spliterator may traverse elements individually (.tryAdvance tryAdvance()) or sequentially in bulk
    (.forEachRemaining forEachRemaining()).
    
    A Spliterator may also partition off some of its elements (using
    .trySplit) as another Spliterator, to be used in
    possibly-parallel operations.  Operations using a Spliterator that
    cannot split, or does so in a highly imbalanced or inefficient
    manner, are unlikely to benefit from parallelism.  Traversal
    and splitting exhaust elements; each Spliterator is useful for only a single
    bulk computation.
    
    A Spliterator also reports a set of .characteristics() of its
    structure, source, and elements from among .ORDERED,
    .DISTINCT, .SORTED, .SIZED, .NONNULL,
    .IMMUTABLE, .CONCURRENT, and .SUBSIZED. These may
    be employed by Spliterator clients to control, specialize or simplify
    computation.  For example, a Spliterator for a Collection would
    report `SIZED`, a Spliterator for a Set would report
    `DISTINCT`, and a Spliterator for a SortedSet would also
    report `SORTED`.  Characteristics are reported as a simple unioned bit
    set.
    
    Some characteristics additionally constrain method behavior; for example if
    `ORDERED`, traversal methods must conform to their documented ordering.
    New characteristics may be defined in the future, so implementors should not
    assign meanings to unlisted values.
    
    <a id="binding">A Spliterator that does not report `IMMUTABLE` or
    `CONCURRENT` is expected to have a documented policy concerning:
    when the spliterator *binds* to the element source; and detection of
    structural interference of the element source detected after binding.</a>  A
    *late-binding* Spliterator binds to the source of elements at the
    point of first traversal, first split, or first query for estimated size,
    rather than at the time the Spliterator is created.  A Spliterator that is
    not *late-binding* binds to the source of elements at the point of
    construction or first invocation of any method.  Modifications made to the
    source prior to binding are reflected when the Spliterator is traversed.
    After binding a Spliterator should, on a best-effort basis, throw
    ConcurrentModificationException if structural interference is
    detected.  Spliterators that do this are called *fail-fast*.  The
    bulk traversal method (.forEachRemaining forEachRemaining()) of a
    Spliterator may optimize traversal and check for structural interference
    after all elements have been traversed, rather than checking per-element and
    failing immediately.
    
    Spliterators can provide an estimate of the number of remaining elements
    via the .estimateSize method.  Ideally, as reflected in characteristic
    .SIZED, this value corresponds exactly to the number of elements
    that would be encountered in a successful traversal.  However, even when not
    exactly known, an estimated value may still be useful to operations
    being performed on the source, such as helping to determine whether it is
    preferable to split further or traverse the remaining elements sequentially.
    
    Despite their obvious utility in parallel algorithms, spliterators are not
    expected to be thread-safe; instead, implementations of parallel algorithms
    using spliterators should ensure that the spliterator is only used by one
    thread at a time.  This is generally easy to attain via *serial
    thread-confinement*, which often is a natural consequence of typical
    parallel algorithms that work by recursive decomposition.  A thread calling
    .trySplit() may hand over the returned Spliterator to another thread,
    which in turn may traverse or further split that Spliterator.  The behaviour
    of splitting and traversal is undefined if two or more threads operate
    concurrently on the same spliterator.  If the original thread hands a
    spliterator off to another thread for processing, it is best if that handoff
    occurs before any elements are consumed with .tryAdvance(Consumer)
    tryAdvance(), as certain guarantees (such as the accuracy of
    .estimateSize() for `SIZED` spliterators) are only valid before
    traversal has begun.
    
    Primitive subtype specializations of `Spliterator` are provided for
    OfInt int, OfLong long, and OfDouble double values.
    The subtype default implementations of
    Spliterator.tryAdvance(java.util.function.Consumer)
    and Spliterator.forEachRemaining(java.util.function.Consumer) box
    primitive values to instances of their corresponding wrapper class.  Such
    boxing may undermine any performance advantages gained by using the primitive
    specializations.  To avoid boxing, the corresponding primitive-based methods
    should be used.  For example,
    Spliterator.OfInt.tryAdvance(java.util.function.IntConsumer)
    and Spliterator.OfInt.forEachRemaining(java.util.function.IntConsumer)
    should be used in preference to
    Spliterator.OfInt.tryAdvance(java.util.function.Consumer) and
    Spliterator.OfInt.forEachRemaining(java.util.function.Consumer).
    Traversal of primitive values using boxing-based methods
    .tryAdvance tryAdvance() and
    .forEachRemaining(java.util.function.Consumer) forEachRemaining()
    does not affect the order in which the values, transformed to boxed values,
    are encountered.
    
    Type `<T>`: the type of elements returned by this Spliterator

    See
    - Collection

    Since
    - 1.8

    Unknown Tags
    - Spliterators, like `Iterator`s, are for traversing the elements of
    a source.  The `Spliterator` API was designed to support efficient
    parallel traversal in addition to sequential traversal, by supporting
    decomposition as well as single-element iteration.  In addition, the
    protocol for accessing elements via a Spliterator is designed to impose
    smaller per-element overhead than `Iterator`, and to avoid the inherent
    race involved in having separate methods for `hasNext()` and
    `next()`.
    
    For mutable sources, arbitrary and non-deterministic behavior may occur if
    the source is structurally interfered with (elements added, replaced, or
    removed) between the time that the Spliterator binds to its data source and
    the end of traversal.  For example, such interference will produce arbitrary,
    non-deterministic results when using the `java.util.stream` framework.
    
    Structural interference of a source can be managed in the following ways
    (in approximate order of decreasing desirability):
    
    - The source cannot be structurally interfered with.
    For example, an instance of
    java.util.concurrent.CopyOnWriteArrayList is an immutable source.
    A Spliterator created from the source reports a characteristic of
    `IMMUTABLE`.
    - The source manages concurrent modifications.
    For example, a key set of a java.util.concurrent.ConcurrentHashMap
    is a concurrent source.  A Spliterator created from the source reports a
    characteristic of `CONCURRENT`.
    - The mutable source provides a late-binding and fail-fast Spliterator.
    Late binding narrows the window during which interference can affect
    the calculation; fail-fast detects, on a best-effort basis, that structural
    interference has occurred after traversal has commenced and throws
    ConcurrentModificationException.  For example, ArrayList,
    and many other non-concurrent `Collection` classes in the JDK, provide
    a late-binding, fail-fast spliterator.
    - The mutable source provides a non-late-binding but fail-fast Spliterator.
    The source increases the likelihood of throwing
    `ConcurrentModificationException` since the window of potential
    interference is larger.
    - The mutable source provides a late-binding and non-fail-fast Spliterator.
    The source risks arbitrary, non-deterministic behavior after traversal
    has commenced since interference is not detected.
    
    - The mutable source provides a non-late-binding and non-fail-fast
    Spliterator.
    The source increases the risk of arbitrary, non-deterministic behavior
    since non-detected interference may occur after construction.
    
    
    
    **Example.** Here is a class (not a very useful one, except
    for illustration) that maintains an array in which the actual data
    are held in even locations, and unrelated tag data are held in odd
    locations. Its Spliterator ignores the tags.
    
    ``` `class TaggedArray<T> {
      private final Object[] elements; // immutable after construction
      TaggedArray(T[] data, Object[] tags) {
        int size = data.length;
        if (tags.length != size) throw new IllegalArgumentException();
        this.elements = new Object[2 * size];
        for (int i = 0, j = 0; i < size; ++i) {
          elements[j++] = data[i];
          elements[j++] = tags[i];`
      }
    
      public Spliterator<T> spliterator() {
        return new TaggedArraySpliterator<>(elements, 0, elements.length);
      }
    
      static class TaggedArraySpliterator<T> implements Spliterator<T> {
        private final Object[] array;
        private int origin; // current index, advanced on split or traversal
        private final int fence; // one past the greatest index
    
        TaggedArraySpliterator(Object[] array, int origin, int fence) {
          this.array = array; this.origin = origin; this.fence = fence;
        }
    
        public void forEachRemaining(Consumer<? super T> action) {
          for (; origin < fence; origin += 2)
            action.accept((T) array[origin]);
        }
    
        public boolean tryAdvance(Consumer<? super T> action) {
          if (origin < fence) {
            action.accept((T) array[origin]);
            origin += 2;
            return True;
          }
          else // cannot advance
            return False;
        }
    
        public Spliterator<T> trySplit() {
          int lo = origin; // divide range in half
          int mid = ((lo + fence) >>> 1) & ~1; // force midpoint to be even
          if (lo < mid) { // split out left half
            origin = mid; // reset this Spliterator's origin
            return new TaggedArraySpliterator<>(array, lo, mid);
          }
          else       // too small to split
            return null;
        }
    
        public long estimateSize() {
          return (long)((fence - origin) / 2);
        }
    
        public int characteristics() {
          return ORDERED | SIZED | IMMUTABLE | SUBSIZED;
        }
      }
    }}```
    
    As an example how a parallel computation framework, such as the
    `java.util.stream` package, would use Spliterator in a parallel
    computation, here is one way to implement an associated parallel forEach,
    that illustrates the primary usage idiom of splitting off subtasks until
    the estimated amount of work is small enough to perform
    sequentially. Here we assume that the order of processing across
    subtasks doesn't matter; different (forked) tasks may further split
    and process elements concurrently in undetermined order.  This
    example uses a java.util.concurrent.CountedCompleter;
    similar usages apply to other parallel task constructions.
    
    ````static <T> void parEach(TaggedArray<T> a, Consumer<T> action) {
      Spliterator<T> s = a.spliterator();
      long targetBatchSize = s.estimateSize() / (ForkJoinPool.getCommonPoolParallelism() * 8);
      new ParEach(null, s, action, targetBatchSize).invoke();`
    
    static class ParEach<T> extends CountedCompleter<Void> {
      final Spliterator<T> spliterator;
      final Consumer<T> action;
      final long targetBatchSize;
    
      ParEach(ParEach<T> parent, Spliterator<T> spliterator,
              Consumer<T> action, long targetBatchSize) {
        super(parent);
        this.spliterator = spliterator; this.action = action;
        this.targetBatchSize = targetBatchSize;
      }
    
      public void compute() {
        Spliterator<T> sub;
        while (spliterator.estimateSize() > targetBatchSize &&
               (sub = spliterator.trySplit()) != null) {
          addToPendingCount(1);
          new ParEach<>(this, sub, action, targetBatchSize).fork();
        }
        spliterator.forEachRemaining(action);
        propagateCompletion();
      }
    }}```
    - If the boolean system property org.openjdk.java.util.stream.tripwire
    is set to `True` then diagnostic warnings are reported if boxing of
    primitive values occur when operating on primitive subtype specializations.
    """

    ORDERED = 0x00000010
    """
    Characteristic value signifying that an encounter order is defined for
    elements. If so, this Spliterator guarantees that method
    .trySplit splits a strict prefix of elements, that method
    .tryAdvance steps by one element in prefix order, and that
    .forEachRemaining performs actions in encounter order.
    
    A Collection has an encounter order if the corresponding
    Collection.iterator documents an order. If so, the encounter
    order is the same as the documented order. Otherwise, a collection does
    not have an encounter order.

    Unknown Tags
    - Encounter order is guaranteed to be ascending index order for
    any List. But no order is guaranteed for hash-based collections
    such as HashSet. Clients of a Spliterator that reports
    `ORDERED` are expected to preserve ordering constraints in
    non-commutative parallel computations.
    """
    DISTINCT = 0x00000001
    """
    Characteristic value signifying that, for each pair of
    encountered elements `x, y`, `!x.equals(y)`. This
    applies for example, to a Spliterator based on a Set.
    """
    SORTED = 0x00000004
    """
    Characteristic value signifying that encounter order follows a defined
    sort order. If so, method .getComparator() returns the associated
    Comparator, or `null` if all elements are Comparable and
    are sorted by their natural ordering.
    
    A Spliterator that reports `SORTED` must also report
    `ORDERED`.

    Unknown Tags
    - The spliterators for `Collection` classes in the JDK that
    implement NavigableSet or SortedSet report `SORTED`.
    """
    SIZED = 0x00000040
    """
    Characteristic value signifying that the value returned from
    `estimateSize()` prior to traversal or splitting represents a
    finite size that, in the absence of structural source modification,
    represents an exact count of the number of elements that would be
    encountered by a complete traversal.

    Unknown Tags
    - Most Spliterators for Collections, that cover all elements of a
    `Collection` report this characteristic. Sub-spliterators, such as
    those for HashSet, that cover a sub-set of elements and
    approximate their reported size do not.
    """
    NONNULL = 0x00000100
    """
    Characteristic value signifying that the source guarantees that
    encountered elements will not be `null`. (This applies,
    for example, to most concurrent collections, queues, and maps.)
    """
    IMMUTABLE = 0x00000400
    """
    Characteristic value signifying that the element source cannot be
    structurally modified; that is, elements cannot be added, replaced, or
    removed, so such changes cannot occur during traversal. A Spliterator
    that does not report `IMMUTABLE` or `CONCURRENT` is expected
    to have a documented policy (for example throwing
    ConcurrentModificationException) concerning structural
    interference detected during traversal.
    """
    CONCURRENT = 0x00001000
    """
    Characteristic value signifying that the element source may be safely
    concurrently modified (allowing additions, replacements, and/or removals)
    by multiple threads without external synchronization. If so, the
    Spliterator is expected to have a documented policy concerning the impact
    of modifications during traversal.
    
    A top-level Spliterator should not report both `CONCURRENT` and
    `SIZED`, since the finite size, if known, may change if the source
    is concurrently modified during traversal. Such a Spliterator is
    inconsistent and no guarantees can be made about any computation using
    that Spliterator. Sub-spliterators may report `SIZED` if the
    sub-split size is known and additions or removals to the source are not
    reflected when traversing.
    
    A top-level Spliterator should not report both `CONCURRENT` and
    `IMMUTABLE`, since they are mutually exclusive. Such a Spliterator
    is inconsistent and no guarantees can be made about any computation using
    that Spliterator. Sub-spliterators may report `IMMUTABLE` if
    additions or removals to the source are not reflected when traversing.

    Unknown Tags
    - Most concurrent collections maintain a consistency policy
    guaranteeing accuracy with respect to elements present at the point of
    Spliterator construction, but possibly not reflecting subsequent
    additions or removals.
    """
    SUBSIZED = 0x00004000
    """
    Characteristic value signifying that all Spliterators resulting from
    `trySplit()` will be both .SIZED and .SUBSIZED.
    (This means that all child Spliterators, whether direct or indirect, will
    be `SIZED`.)
    
    A Spliterator that does not report `SIZED` as required by
    `SUBSIZED` is inconsistent and no guarantees can be made about any
    computation using that Spliterator.

    Unknown Tags
    - Some spliterators, such as the top-level spliterator for an
    approximately balanced binary tree, will report `SIZED` but not
    `SUBSIZED`, since it is common to know the size of the entire tree
    but not the exact sizes of subtrees.
    """


    def tryAdvance(self, action: "Consumer"["T"]) -> bool:
        """
        If a remaining element exists, performs the given action on it,
        returning `True`; else returns `False`.  If this
        Spliterator is .ORDERED the action is performed on the
        next element in encounter order.  Exceptions thrown by the
        action are relayed to the caller.
        
        Subsequent behavior of a spliterator is unspecified if the action throws
        an exception.

        Arguments
        - action: The action

        Returns
        - `False` if no remaining elements existed
        upon entry to this method, else `True`.

        Raises
        - NullPointerException: if the specified action is null
        """
        ...


    def forEachRemaining(self, action: "Consumer"["T"]) -> None:
        """
        Performs the given action for each remaining element, sequentially in
        the current thread, until all elements have been processed or the action
        throws an exception.  If this Spliterator is .ORDERED, actions
        are performed in encounter order.  Exceptions thrown by the action
        are relayed to the caller.
        
        Subsequent behavior of a spliterator is unspecified if the action throws
        an exception.

        Arguments
        - action: The action

        Raises
        - NullPointerException: if the specified action is null

        Unknown Tags
        - The default implementation repeatedly invokes .tryAdvance until
        it returns `False`.  It should be overridden whenever possible.
        """
        ...


    def trySplit(self) -> "Spliterator"["T"]:
        """
        If this spliterator can be partitioned, returns a Spliterator
        covering elements, that will, upon return from this method, not
        be covered by this Spliterator.
        
        If this Spliterator is .ORDERED, the returned Spliterator
        must cover a strict prefix of the elements.
        
        Unless this Spliterator covers an infinite number of elements,
        repeated calls to `trySplit()` must eventually return `null`.
        Upon non-null return:
        
        - the value reported for `estimateSize()` before splitting,
        must, after splitting, be greater than or equal to `estimateSize()`
        for this and the returned Spliterator; and
        - if this Spliterator is `SUBSIZED`, then `estimateSize()`
        for this spliterator before splitting must be equal to the sum of
        `estimateSize()` for this and the returned Spliterator after
        splitting.
        
        
        This method may return `null` for any reason,
        including emptiness, inability to split after traversal has
        commenced, data structure constraints, and efficiency
        considerations.

        Returns
        - a `Spliterator` covering some portion of the
        elements, or `null` if this spliterator cannot be split

        Unknown Tags
        - An ideal `trySplit` method efficiently (without
        traversal) divides its elements exactly in half, allowing
        balanced parallel computation.  Many departures from this ideal
        remain highly effective; for example, only approximately
        splitting an approximately balanced tree, or for a tree in
        which leaf nodes may contain either one or two elements,
        failing to further split these nodes.  However, large
        deviations in balance and/or overly inefficient `trySplit` mechanics typically result in poor parallel
        performance.
        """
        ...


    def estimateSize(self) -> int:
        """
        Returns an estimate of the number of elements that would be
        encountered by a .forEachRemaining traversal, or returns Long.MAX_VALUE if infinite, unknown, or too expensive to compute.
        
        If this Spliterator is .SIZED and has not yet been partially
        traversed or split, or this Spliterator is .SUBSIZED and has
        not yet been partially traversed, this estimate must be an accurate
        count of elements that would be encountered by a complete traversal.
        Otherwise, this estimate may be arbitrarily inaccurate, but must decrease
        as specified across invocations of .trySplit.

        Returns
        - the estimated size, or `Long.MAX_VALUE` if infinite,
                unknown, or too expensive to compute.

        Unknown Tags
        - Even an inexact estimate is often useful and inexpensive to compute.
        For example, a sub-spliterator of an approximately balanced binary tree
        may return a value that estimates the number of elements to be half of
        that of its parent; if the root Spliterator does not maintain an
        accurate count, it could estimate size to be the power of two
        corresponding to its maximum depth.
        """
        ...


    def getExactSizeIfKnown(self) -> int:
        """
        Convenience method that returns .estimateSize() if this
        Spliterator is .SIZED, else `-1`.

        Returns
        - the exact size, if known, else `-1`.

        Unknown Tags
        - The default implementation returns the result of `estimateSize()`
        if the Spliterator reports a characteristic of `SIZED`, and
        `-1` otherwise.
        """
        ...


    def characteristics(self) -> int:
        """
        Returns a set of characteristics of this Spliterator and its
        elements. The result is represented as ORed values from .ORDERED, .DISTINCT, .SORTED, .SIZED,
        .NONNULL, .IMMUTABLE, .CONCURRENT,
        .SUBSIZED.  Repeated calls to `characteristics()` on
        a given spliterator, prior to or in-between calls to `trySplit`,
        should always return the same result.
        
        If a Spliterator reports an inconsistent set of
        characteristics (either those returned from a single invocation
        or across multiple invocations), no guarantees can be made
        about any computation using this Spliterator.

        Returns
        - a representation of characteristics

        Unknown Tags
        - The characteristics of a given spliterator before splitting
        may differ from the characteristics after splitting.  For specific
        examples see the characteristic values .SIZED, .SUBSIZED
        and .CONCURRENT.
        """
        ...


    def hasCharacteristics(self, characteristics: int) -> bool:
        """
        Returns `True` if this Spliterator's .characteristics contain all of the given characteristics.

        Arguments
        - characteristics: the characteristics to check for

        Returns
        - `True` if all the specified characteristics are present,
        else `False`

        Unknown Tags
        - The default implementation returns True if the corresponding bits
        of the given characteristics are set.
        """
        ...


    def getComparator(self) -> "Comparator"["T"]:
        """
        If this Spliterator's source is .SORTED by a Comparator,
        returns that `Comparator`. If the source is `SORTED` in
        Comparable natural order, returns `null`.  Otherwise,
        if the source is not `SORTED`, throws IllegalStateException.

        Returns
        - a Comparator, or `null` if the elements are sorted in the
        natural order.

        Raises
        - IllegalStateException: if the spliterator does not report
                a characteristic of `SORTED`.

        Unknown Tags
        - The default implementation always throws IllegalStateException.
        """
        ...


    class OfPrimitive(Spliterator):
        """
        A Spliterator specialized for primitive values.
        
        Type `<T>`: the type of elements returned by this Spliterator.  The
        type must be a wrapper type for a primitive type, such as `Integer`
        for the primitive `int` type.

        Arguments
        - <T_CONS>: the type of primitive consumer.  The type must be a
        primitive specialization of java.util.function.Consumer for
        `T`, such as java.util.function.IntConsumer for
        `Integer`.
        - <T_SPLITR>: the type of primitive Spliterator.  The type must be
        a primitive specialization of Spliterator for `T`, such as
        Spliterator.OfInt for `Integer`.

        See
        - Spliterator.OfDouble

        Since
        - 1.8
        """

        def trySplit(self) -> "T_SPLITR":
            ...


        def tryAdvance(self, action: "T_CONS") -> bool:
            """
            If a remaining element exists, performs the given action on it,
            returning `True`; else returns `False`.  If this
            Spliterator is .ORDERED the action is performed on the
            next element in encounter order.  Exceptions thrown by the
            action are relayed to the caller.
            
            Subsequent behavior of a spliterator is unspecified if the action throws
            an exception.

            Arguments
            - action: The action

            Returns
            - `False` if no remaining elements existed
            upon entry to this method, else `True`.

            Raises
            - NullPointerException: if the specified action is null
            """
            ...


        def forEachRemaining(self, action: "T_CONS") -> None:
            """
            Performs the given action for each remaining element, sequentially in
            the current thread, until all elements have been processed or the
            action throws an exception.  If this Spliterator is .ORDERED,
            actions are performed in encounter order.  Exceptions thrown by the
            action are relayed to the caller.
            
            Subsequent behavior of a spliterator is unspecified if the action throws
            an exception.

            Arguments
            - action: The action

            Raises
            - NullPointerException: if the specified action is null

            Unknown Tags
            - The default implementation repeatedly invokes .tryAdvance
            until it returns `False`.  It should be overridden whenever
            possible.
            """
            ...


    class OfInt(OfPrimitive):
        """
        A Spliterator specialized for `int` values.

        Since
        - 1.8
        """

        def trySplit(self) -> "OfInt":
            ...


        def tryAdvance(self, action: "IntConsumer") -> bool:
            ...


        def forEachRemaining(self, action: "IntConsumer") -> None:
            ...


        def tryAdvance(self, action: "Consumer"["Integer"]) -> bool:
            """
            Unknown Tags
            - If the action is an instance of `IntConsumer` then it is cast
            to `IntConsumer` and passed to
            .tryAdvance(java.util.function.IntConsumer); otherwise
            the action is adapted to an instance of `IntConsumer`, by
            boxing the argument of `IntConsumer`, and then passed to
            .tryAdvance(java.util.function.IntConsumer).
            """
            ...


        def forEachRemaining(self, action: "Consumer"["Integer"]) -> None:
            """
            Unknown Tags
            - If the action is an instance of `IntConsumer` then it is cast
            to `IntConsumer` and passed to
            .forEachRemaining(java.util.function.IntConsumer); otherwise
            the action is adapted to an instance of `IntConsumer`, by
            boxing the argument of `IntConsumer`, and then passed to
            .forEachRemaining(java.util.function.IntConsumer).
            """
            ...


    class OfLong(OfPrimitive):
        """
        A Spliterator specialized for `long` values.

        Since
        - 1.8
        """

        def trySplit(self) -> "OfLong":
            ...


        def tryAdvance(self, action: "LongConsumer") -> bool:
            ...


        def forEachRemaining(self, action: "LongConsumer") -> None:
            ...


        def tryAdvance(self, action: "Consumer"["Long"]) -> bool:
            """
            Unknown Tags
            - If the action is an instance of `LongConsumer` then it is cast
            to `LongConsumer` and passed to
            .tryAdvance(java.util.function.LongConsumer); otherwise
            the action is adapted to an instance of `LongConsumer`, by
            boxing the argument of `LongConsumer`, and then passed to
            .tryAdvance(java.util.function.LongConsumer).
            """
            ...


        def forEachRemaining(self, action: "Consumer"["Long"]) -> None:
            """
            Unknown Tags
            - If the action is an instance of `LongConsumer` then it is cast
            to `LongConsumer` and passed to
            .forEachRemaining(java.util.function.LongConsumer); otherwise
            the action is adapted to an instance of `LongConsumer`, by
            boxing the argument of `LongConsumer`, and then passed to
            .forEachRemaining(java.util.function.LongConsumer).
            """
            ...


    class OfDouble(OfPrimitive):
        """
        A Spliterator specialized for `double` values.

        Since
        - 1.8
        """

        def trySplit(self) -> "OfDouble":
            ...


        def tryAdvance(self, action: "DoubleConsumer") -> bool:
            ...


        def forEachRemaining(self, action: "DoubleConsumer") -> None:
            ...


        def tryAdvance(self, action: "Consumer"["Double"]) -> bool:
            """
            Unknown Tags
            - If the action is an instance of `DoubleConsumer` then it is
            cast to `DoubleConsumer` and passed to
            .tryAdvance(java.util.function.DoubleConsumer); otherwise
            the action is adapted to an instance of `DoubleConsumer`, by
            boxing the argument of `DoubleConsumer`, and then passed to
            .tryAdvance(java.util.function.DoubleConsumer).
            """
            ...


        def forEachRemaining(self, action: "Consumer"["Double"]) -> None:
            """
            Unknown Tags
            - If the action is an instance of `DoubleConsumer` then it is
            cast to `DoubleConsumer` and passed to
            .forEachRemaining(java.util.function.DoubleConsumer);
            otherwise the action is adapted to an instance of
            `DoubleConsumer`, by boxing the argument of
            `DoubleConsumer`, and then passed to
            .forEachRemaining(java.util.function.DoubleConsumer).
            """
            ...



"""
Python module generated from Java source file java.lang.reflect.InvocationTargetException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import *
from typing import Any, Callable, Iterable, Tuple


class InvocationTargetException(ReflectiveOperationException):
    """
    InvocationTargetException is a checked exception that wraps
    an exception thrown by an invoked method or constructor.

    See
    - Constructor

    Since
    - 1.1
    """

    def __init__(self, target: "Throwable"):
        """
        Constructs a InvocationTargetException with a target exception.

        Arguments
        - target: the target exception
        """
        ...


    def __init__(self, target: "Throwable", s: str):
        """
        Constructs a InvocationTargetException with a target exception
        and a detail message.

        Arguments
        - target: the target exception
        - s: the detail message
        """
        ...


    def getTargetException(self) -> "Throwable":
        """
        Get the thrown target exception.

        Returns
        - the thrown target exception (cause of this exception).

        Unknown Tags
        - This method predates the general-purpose exception chaining facility.
        The Throwable.getCause() method is now the preferred means of
        obtaining this information.
        """
        ...


    def getCause(self) -> "Throwable":
        """
        Returns the cause of this exception (the thrown target exception,
        which may be `null`).

        Returns
        - the cause of this exception.

        Since
        - 1.4
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.tags.ItemTagType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory.meta.tags import *
from org.bukkit.persistence import PersistentDataType
from typing import Any, Callable, Iterable, Tuple


class ItemTagType:
    """
    This class represents an enum with a generic content type. It defines the
    types a custom item tag can have.
    
    This interface can be used to create your own custom ItemTagType with
    different complex types. This may be useful for the likes of a
    UUIDItemTagType:
    ```
    `public class UUIDItemTagType implements ItemTagType<byte[], UUID> {
    
            {@literal @Override`
            public Class<byte[]> getPrimitiveType() {
                return byte[].class;
            }
    
            @Override
            public Class<UUID> getComplexType() {
                return UUID.class;
            }
    
            @Override
            public byte[] toPrimitive(UUID complex, ItemTagAdapterContext context) {
                ByteBuffer bb = ByteBuffer.wrap(new byte[16]);
                bb.putLong(complex.getMostSignificantBits());
                bb.putLong(complex.getLeastSignificantBits());
                return bb.array();
            }
    
            @Override
            public UUID fromPrimitive(byte[] primitive, ItemTagAdapterContext context) {
                ByteBuffer bb = ByteBuffer.wrap(primitive);
                long firstLong = bb.getLong();
                long secondLong = bb.getLong();
                return new UUID(firstLong, secondLong);
            }
        }}```
    
    Type `<T>`: the primary object type that is stored in the given tag
    
    Type `<Z>`: the retrieved object type when applying this item tag type

    Deprecated
    - please use PersistentDataType as this part of the api is being replaced
    """

    BYTE = PrimitiveTagType<>(Byte.class)
    SHORT = PrimitiveTagType<>(Short.class)
    INTEGER = PrimitiveTagType<>(Integer.class)
    LONG = PrimitiveTagType<>(Long.class)
    FLOAT = PrimitiveTagType<>(Float.class)
    DOUBLE = PrimitiveTagType<>(Double.class)
    STRING = PrimitiveTagType<>(String.class)
    BYTE_ARRAY = PrimitiveTagType<>(byte[].class)
    INTEGER_ARRAY = PrimitiveTagType<>(int[].class)
    LONG_ARRAY = PrimitiveTagType<>(long[].class)
    TAG_CONTAINER = PrimitiveTagType<>(CustomItemTagContainer.class)


    def getPrimitiveType(self) -> type["T"]:
        """
        Returns the primitive data type of this tag.

        Returns
        - the class
        """
        ...


    def getComplexType(self) -> type["Z"]:
        """
        Returns the complex object type the primitive value resembles.

        Returns
        - the class type
        """
        ...


    def toPrimitive(self, complex: "Z", context: "ItemTagAdapterContext") -> "T":
        """
        Returns the primitive data that resembles the complex object passed to
        this method.

        Arguments
        - complex: the complex object instance
        - context: the context this operation is running in

        Returns
        - the primitive value
        """
        ...


    def fromPrimitive(self, primitive: "T", context: "ItemTagAdapterContext") -> "Z":
        """
        Creates a complex object based of the passed primitive value

        Arguments
        - primitive: the primitive value
        - context: the context this operation is running in

        Returns
        - the complex object instance
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.server.PluginEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event.server import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class PluginEvent(ServerEvent):
    """
    Used for plugin enable and disable events
    """

    def __init__(self, plugin: "Plugin"):
        ...


    def getPlugin(self) -> "Plugin":
        """
        Gets the plugin involved in this event

        Returns
        - Plugin for this event
        """
        ...
